@@ -12,7 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import hashlib
 import io
 import os
 import re
@@ -24,11 +23,13 @@ import flask
 import jinja2
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils.secretutils import md5
 import webob
 from werkzeug import exceptions
 
 from octavia.amphorae.backends.agent.api_server import haproxy_compatibility
 from octavia.amphorae.backends.agent.api_server import util
+from octavia.amphorae.backends.utils import haproxy_query
 from octavia.common import constants as consts
 from octavia.common import utils as octavia_utils
 
@@ -55,7 +56,7 @@ SYSTEMD_TEMPLATE = JINJA_ENV.get_template(SYSTEMD_CONF)
 class Wrapped(object):
     def __init__(self, stream_):
         self.stream = stream_
-        self.hash = hashlib.md5()  # nosec
+        self.hash = md5(usedforsecurity=False)  # nosec
 
     def read(self, line):
         block = self.stream.read(line)
@@ -78,11 +79,12 @@ class Loadbalancer(object):
         :param listener_id: the id of the listener
         """
         self._check_lb_exists(lb_id)
-        with open(util.config_path(lb_id), 'r') as file:
+        with open(util.config_path(lb_id), 'r', encoding='utf-8') as file:
             cfg = file.read()
             resp = webob.Response(cfg, content_type='text/plain')
             resp.headers['ETag'] = (
-                hashlib.md5(octavia_utils.b(cfg)).hexdigest())  # nosec
+                md5(octavia_utils.b(cfg),
+                    usedforsecurity=False).hexdigest())  # nosec
             return resp
 
     def upload_haproxy_config(self, amphora_id, lb_id):
@@ -188,6 +190,8 @@ class Loadbalancer(object):
                     haproxy_pid=util.pid_path(lb_id),
                     haproxy_cmd=util.CONF.haproxy_amphora.haproxy_cmd,
                     haproxy_cfg=util.config_path(lb_id),
+                    haproxy_state_file=util.state_file_path(lb_id),
+                    haproxy_socket=util.haproxy_sock_path(lb_id),
                     haproxy_user_group_cfg=consts.HAPROXY_USER_GROUP_CFG,
                     respawn_count=util.CONF.haproxy_amphora.respawn_count,
                     respawn_interval=(util.CONF.haproxy_amphora.
@@ -241,6 +245,17 @@ class Loadbalancer(object):
         if action == consts.AMP_ACTION_RELOAD:
             if consts.OFFLINE == self._check_haproxy_status(lb_id):
                 action = consts.AMP_ACTION_START
+            else:
+                # We first have to save the state when we reload
+                haproxy_state_file = util.state_file_path(lb_id)
+                stat_sock_file = util.haproxy_sock_path(lb_id)
+
+                lb_query = haproxy_query.HAProxyQuery(stat_sock_file)
+                if not lb_query.save_state(haproxy_state_file):
+                    # We accept to reload haproxy even if the state_file is
+                    # not generated, but we probably want to know about that
+                    # failure!
+                    LOG.warning('Failed to save haproxy-%s state!', lb_id)
 
         cmd = ("/usr/sbin/service haproxy-{lb_id} {action}".format(
             lb_id=lb_id, action=action))
@@ -359,7 +374,7 @@ class Loadbalancer(object):
 
         Currently type==SSL is also not detected
         """
-        listeners = list()
+        listeners = []
 
         for lb in util.get_loadbalancers():
             stats_socket, listeners_on_lb = util.parse_haproxy_file(lb)
@@ -384,7 +399,7 @@ class Loadbalancer(object):
 
         stream = Wrapped(flask.request.stream)
         file = self._cert_file_path(lb_id, filename)
-        flags = os.O_WRONLY | os.O_CREAT
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
         # mode 00600
         mode = stat.S_IRUSR | stat.S_IWUSR
         with os.fdopen(os.open(file, flags, mode), 'wb') as crt_file:
@@ -408,11 +423,12 @@ class Loadbalancer(object):
                 details="No certificate with filename: {f}".format(
                     f=filename)), status=404)
 
-        with open(cert_path, 'r') as crt_file:
+        with open(cert_path, 'r', encoding='utf-8') as crt_file:
             cert = crt_file.read()
-            md5 = hashlib.md5(octavia_utils.b(cert)).hexdigest()  # nosec
-            resp = webob.Response(json=dict(md5sum=md5))
-            resp.headers['ETag'] = md5
+            md5sum = md5(octavia_utils.b(cert),
+                         usedforsecurity=False).hexdigest()  # nosec
+            resp = webob.Response(json=dict(md5sum=md5sum))
+            resp.headers['ETag'] = md5sum
             return resp
 
     def delete_certificate(self, lb_id, filename):
@@ -426,7 +442,8 @@ class Loadbalancer(object):
             if os.path.exists(
                     os.path.join('/proc', util.get_haproxy_pid(lb_id))):
                 # Check if the listener is disabled
-                with open(util.config_path(lb_id), 'r') as file:
+                with open(util.config_path(lb_id),
+                          'r', encoding='utf-8') as file:
                     cfg = file.read()
                     m = re.findall('^frontend (.*)$', cfg, re.MULTILINE)
                     return m or []
