@@ -52,6 +52,13 @@ AMPHORA_SUPPORTED_PROTOCOLS = [
     lib_consts.PROTOCOL_PROXY,
     lib_consts.PROTOCOL_PROXYV2,
     lib_consts.PROTOCOL_UDP,
+    lib_consts.PROTOCOL_SCTP,
+    lib_consts.PROTOCOL_PROMETHEUS,
+]
+
+VALID_L7POLICY_LISTENER_PROTOCOLS = [
+    lib_consts.PROTOCOL_HTTP,
+    lib_consts.PROTOCOL_TERMINATED_HTTPS
 ]
 
 
@@ -83,11 +90,11 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 user_fault_string=msg,
                 operator_fault_string=msg)
 
-    def _validate_alpn_protocols(self, listener):
-        if not listener.alpn_protocols:
+    def _validate_alpn_protocols(self, obj):
+        if not obj.alpn_protocols:
             return
         supported = consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS
-        not_supported = set(listener.alpn_protocols) - set(supported)
+        not_supported = set(obj.alpn_protocols) - set(supported)
         if not_supported:
             msg = ('Amphora provider does not support %s ALPN protocol(s). '
                    'Supported: %s'
@@ -97,7 +104,13 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 operator_fault_string=msg)
 
     # Load Balancer
-    def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary):
+    def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary,
+                        additional_vip_dicts=None):
+        if additional_vip_dicts:
+            msg = ('Amphora v1 driver does not support additional_vips.')
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
         vip_obj = driver_utils.provider_vip_dict_to_vip_obj(vip_dictionary)
         lb_obj = data_models.LoadBalancer(id=loadbalancer_id,
                                           project_id=project_id, vip=vip_obj)
@@ -111,7 +124,9 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                                          operator_fault_string=message)
 
         try:
-            vip = network_driver.allocate_vip(lb_obj)
+            # allocated_vip returns (vip, add_vips), skipping the 2nd element
+            # as amphorav1 doesn't support add_vips
+            vip = network_driver.allocate_vip(lb_obj)[0]
         except network_base.AllocateVIPException as e:
             message = str(e)
             if getattr(e, 'orig_msg', None) is not None:
@@ -121,7 +136,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
         LOG.info('Amphora provider created VIP port %s for load balancer %s.',
                  vip.port_id, loadbalancer_id)
-        return driver_utils.vip_dict_to_provider_dict(vip.to_dict())
+        return driver_utils.vip_dict_to_provider_dict(vip.to_dict()), []
 
     # TODO(johnsom) convert this to octavia_lib constant flavor
     # once octavia is transitioned to use octavia_lib
@@ -196,6 +211,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
     # Pool
     def pool_create(self, pool):
         self._validate_pool_algorithm(pool)
+        self._validate_alpn_protocols(pool)
         payload = {consts.POOL_ID: pool.pool_id}
         self.client.cast({}, 'create_pool', **payload)
 
@@ -205,6 +221,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         self.client.cast({}, 'delete_pool', **payload)
 
     def pool_update(self, old_pool, new_pool):
+        self._validate_alpn_protocols(new_pool)
         if new_pool.lb_algorithm:
             self._validate_pool_algorithm(new_pool)
         pool_dict = new_pool.to_dict()
@@ -291,8 +308,8 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         self.client.cast({}, 'batch_update_members', **payload)
 
     def _validate_members(self, db_pool, members):
-        if db_pool.protocol == consts.PROTOCOL_UDP:
-            # For UDP LBs, check that we are not mixing IPv4 and IPv6
+        if db_pool.protocol in consts.LVS_PROTOCOLS:
+            # For SCTP/UDP LBs, check that we are not mixing IPv4 and IPv6
             for member in members:
                 member_is_ipv6 = utils.is_ipv6(member.address)
 
@@ -302,8 +319,8 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
                     if member_is_ipv6 != vip_is_ipv6:
                         msg = ("This provider doesn't support mixing IPv4 and "
-                               "IPv6 addresses for its VIP and members in UDP "
-                               "load balancers.")
+                               "IPv6 addresses for its VIP and members in {} "
+                               "load balancers.".format(db_pool.protocol))
                         raise exceptions.UnsupportedOptionError(
                             user_fault_string=msg,
                             operator_fault_string=msg)
@@ -336,6 +353,14 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
     # L7 Policy
     def l7policy_create(self, l7policy):
+        db_listener = self.repositories.listener.get(db_apis.get_session(),
+                                                     id=l7policy.listener_id)
+        if db_listener.protocol not in VALID_L7POLICY_LISTENER_PROTOCOLS:
+            msg = ('%s protocol listeners do not support L7 policies' % (
+                db_listener.protocol))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
         payload = {consts.L7POLICY_ID: l7policy.l7policy_id}
         self.client.cast({}, 'create_l7policy', **payload)
 
