@@ -24,6 +24,7 @@ from stevedore import driver as stevedore_driver
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
 from octavia_lib.api.drivers import provider_base as driver_base
+from octavia_lib.common import constants as lib_consts
 
 from octavia.api.drivers.amphora_driver import availability_zone_schema
 from octavia.api.drivers.amphora_driver import flavor_schema
@@ -44,6 +45,23 @@ AMPHORA_SUPPORTED_LB_ALGORITHMS = [
     consts.LB_ALGORITHM_SOURCE_IP,
     consts.LB_ALGORITHM_LEAST_CONNECTIONS]
 
+AMPHORA_SUPPORTED_PROTOCOLS = [
+    lib_consts.PROTOCOL_TCP,
+    lib_consts.PROTOCOL_HTTP,
+    lib_consts.PROTOCOL_HTTPS,
+    lib_consts.PROTOCOL_TERMINATED_HTTPS,
+    lib_consts.PROTOCOL_PROXY,
+    lib_consts.PROTOCOL_PROXYV2,
+    lib_consts.PROTOCOL_UDP,
+    lib_consts.PROTOCOL_SCTP,
+    lib_consts.PROTOCOL_PROMETHEUS,
+]
+
+VALID_L7POLICY_LISTENER_PROTOCOLS = [
+    lib_consts.PROTOCOL_HTTP,
+    lib_consts.PROTOCOL_TERMINATED_HTTPS
+]
+
 
 class AmphoraProviderDriver(driver_base.ProviderDriver):
     def __init__(self):
@@ -64,11 +82,21 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 user_fault_string=msg,
                 operator_fault_string=msg)
 
-    def _validate_alpn_protocols(self, listener):
-        if not listener.alpn_protocols:
+    def _validate_listener_protocol(self, listener):
+        if listener.protocol not in AMPHORA_SUPPORTED_PROTOCOLS:
+            msg = ('Amphora provider does not support %s protocol. '
+                   'Supported: %s'
+                   % (listener.protocol,
+                      ", ".join(AMPHORA_SUPPORTED_PROTOCOLS)))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+
+    def _validate_alpn_protocols(self, obj):
+        if not obj.alpn_protocols:
             return
         supported = consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS
-        not_supported = set(listener.alpn_protocols) - set(supported)
+        not_supported = set(obj.alpn_protocols) - set(supported)
         if not_supported:
             msg = ('Amphora provider does not support %s ALPN protocol(s). '
                    'Supported: %s'
@@ -78,14 +106,27 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 operator_fault_string=msg)
 
     # Load Balancer
-    def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary):
+    def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary,
+                        additional_vip_dicts):
         vip_obj = driver_utils.provider_vip_dict_to_vip_obj(vip_dictionary)
+        add_vip_objs = [
+            driver_utils.provider_additional_vip_dict_to_additional_vip_obj(
+                add_vip)
+            for add_vip in additional_vip_dicts]
         lb_obj = data_models.LoadBalancer(id=loadbalancer_id,
-                                          project_id=project_id, vip=vip_obj)
+                                          project_id=project_id, vip=vip_obj,
+                                          additional_vips=add_vip_objs)
 
         network_driver = utils.get_network_driver()
+        vip_network = network_driver.get_network(
+            vip_dictionary[lib_consts.VIP_NETWORK_ID])
+        if not vip_network.port_security_enabled:
+            message = "Port security must be enabled on the VIP network."
+            raise exceptions.DriverError(user_fault_string=message,
+                                         operator_fault_string=message)
+
         try:
-            vip = network_driver.allocate_vip(lb_obj)
+            vip, add_vips = network_driver.allocate_vip(lb_obj)
         except network_base.AllocateVIPException as e:
             message = str(e)
             if getattr(e, 'orig_msg', None) is not None:
@@ -95,7 +136,10 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
         LOG.info('Amphora provider created VIP port %s for load balancer %s.',
                  vip.port_id, loadbalancer_id)
-        return driver_utils.vip_dict_to_provider_dict(vip.to_dict())
+        vip_return_dict = driver_utils.vip_dict_to_provider_dict(vip.to_dict())
+        add_return_dicts = [driver_utils.additional_vip_dict_to_provider_dict(
+            add_vip.to_dict()) for add_vip in add_vips]
+        return vip_return_dict, add_return_dicts
 
     # TODO(johnsom) convert this to octavia_lib constant flavor
     # once octavia is transitioned to use octavia_lib
@@ -165,6 +209,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
     # Listener
     def listener_create(self, listener):
+        self._validate_listener_protocol(listener)
         self._validate_alpn_protocols(listener)
         payload = {consts.LISTENER: listener.to_dict()}
         self._encrypt_listener_dict(payload[consts.LISTENER])
@@ -193,20 +238,21 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         if 'admin_state_up' in pool_dict:
             pool_dict['enabled'] = pool_dict.pop('admin_state_up')
         if 'tls_container_ref' in pool_dict:
-            pool_dict['tls_container_id'] = pool_dict.pop('tls_container_ref')
+            pool_dict['tls_certificate_id'] = pool_dict.pop(
+                'tls_container_ref')
         pool_dict.pop('tls_container_data', None)
         if 'ca_tls_container_ref' in pool_dict:
             pool_dict['ca_tls_certificate_id'] = pool_dict.pop(
                 'ca_tls_container_ref')
         pool_dict.pop('ca_tls_container_data', None)
-        if 'client_crl_container_ref' in pool_dict:
-            pool_dict['client_crl_container_id'] = pool_dict.pop(
-                'client_crl_container_ref')
-        pool_dict.pop('client_crl_container_data', None)
+        if 'crl_container_ref' in pool_dict:
+            pool_dict['crl_container_id'] = pool_dict.pop('crl_container_ref')
+        pool_dict.pop('crl_container_data', None)
         return pool_dict
 
     def pool_create(self, pool):
         self._validate_pool_algorithm(pool)
+        self._validate_alpn_protocols(pool)
         payload = {consts.POOL: self._pool_convert_to_dict(pool)}
         self.client.cast({}, 'create_pool', **payload)
 
@@ -215,6 +261,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         self.client.cast({}, 'delete_pool', **payload)
 
     def pool_update(self, old_pool, new_pool):
+        self._validate_alpn_protocols(new_pool)
         if new_pool.lb_algorithm:
             self._validate_pool_algorithm(new_pool)
         pool_dict = self._pool_convert_to_dict(new_pool)
@@ -280,28 +327,33 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             if m.id not in new_member_ids:
                 deleted_members.append(m)
 
-        if deleted_members or new_members or updated_members:
-            payload = {'old_members': [m.to_dict() for m in deleted_members],
-                       'new_members': [m.to_dict() for m in new_members],
-                       'updated_members': updated_members}
-            self.client.cast({}, 'batch_update_members', **payload)
-        else:
-            LOG.info("Member batch update is a noop, returning early.")
+        payload = {'old_members': [m.to_dict() for m in deleted_members],
+                   'new_members': [m.to_dict() for m in new_members],
+                   'updated_members': updated_members}
+        self.client.cast({}, 'batch_update_members', **payload)
 
     def _validate_members(self, db_pool, members):
-        if db_pool.protocol == consts.PROTOCOL_UDP:
-            # For UDP LBs, check that we are not mixing IPv4 and IPv6
+        if db_pool.protocol in consts.LVS_PROTOCOLS:
+            # For SCTP/UDP LBs:
+            # Allow ipv4 member if there's at least one ipv4 VIP
+            # Allow ipv6 member if there's at least one ipv6 VIP
             for member in members:
                 member_is_ipv6 = utils.is_ipv6(member.address)
 
                 for listener in db_pool.listeners:
                     lb = listener.load_balancer
-                    vip_is_ipv6 = utils.is_ipv6(lb.vip.ip_address)
+                    vips = [lb.vip]
+                    vips.extend(lb.additional_vips)
+                    lb_has_ipv4 = any(utils.is_ipv4(vip.ip_address)
+                                      for vip in vips)
+                    lb_has_ipv6 = any(utils.is_ipv6(vip.ip_address)
+                                      for vip in vips)
 
-                    if member_is_ipv6 != vip_is_ipv6:
+                    if ((member_is_ipv6 and not lb_has_ipv6) or
+                            (not member_is_ipv6 and not lb_has_ipv4)):
                         msg = ("This provider doesn't support mixing IPv4 and "
-                               "IPv6 addresses for its VIP and members in UDP "
-                               "load balancers.")
+                               "IPv6 addresses for its VIP and members in {} "
+                               "load balancers.".format(db_pool.protocol))
                         raise exceptions.UnsupportedOptionError(
                             user_fault_string=msg,
                             operator_fault_string=msg)
@@ -333,6 +385,14 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
     # L7 Policy
     def l7policy_create(self, l7policy):
+        db_listener = self.repositories.listener.get(db_apis.get_session(),
+                                                     id=l7policy.listener_id)
+        if db_listener.protocol not in VALID_L7POLICY_LISTENER_PROTOCOLS:
+            msg = ('%s protocol listeners do not support L7 policies' % (
+                db_listener.protocol))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
         payload = {consts.L7POLICY: l7policy.to_dict()}
         self.client.cast({}, 'create_l7policy', **payload)
 
