@@ -14,8 +14,6 @@
 
 import copy
 
-import six
-
 from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions as lib_exceptions
 from oslo_config import cfg
@@ -24,7 +22,7 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from stevedore import driver as stevedore_driver
 
-from octavia.api.drivers import exceptions as driver_exceptions
+from octavia.common import constants
 from octavia.common import data_models
 from octavia.common import exceptions
 from octavia.common.tls_utils import cert_parser
@@ -52,14 +50,12 @@ def call_provider(provider, driver_method, *args, **kwargs):
 
     try:
         return driver_method(*args, **kwargs)
-    except (driver_exceptions.DriverError, lib_exceptions.DriverError) as e:
+    except lib_exceptions.DriverError as e:
         LOG.exception("Provider '%s' raised a driver error: %s",
                       provider, e.operator_fault_string)
         raise exceptions.ProviderDriverError(prov=provider,
                                              user_msg=e.user_fault_string)
-    except (driver_exceptions.NotImplementedError,
-            lib_exceptions.NotImplementedError,
-            NotImplementedError) as e:
+    except (lib_exceptions.NotImplementedError, NotImplementedError) as e:
         op_fault_string = (
             e.operator_fault_string
             if hasattr(e, "operator_fault_string")
@@ -72,15 +68,14 @@ def call_provider(provider, driver_method, *args, **kwargs):
                  provider, op_fault_string)
         raise exceptions.ProviderNotImplementedError(
             prov=provider, user_msg=usr_fault_string)
-    except (driver_exceptions.UnsupportedOptionError,
-            lib_exceptions.UnsupportedOptionError) as e:
+    except lib_exceptions.UnsupportedOptionError as e:
         LOG.info("Provider '%s' raised an unsupported option error: "
                  "%s", provider, e.operator_fault_string)
         raise exceptions.ProviderUnsupportedOptionError(
             prov=provider, user_msg=e.user_fault_string)
     except Exception as e:
         LOG.exception("Provider '%s' raised an unknown error: %s",
-                      provider, e)
+                      provider, str(e))
         raise exceptions.ProviderDriverError(prov=provider, user_msg=e)
 
 
@@ -125,7 +120,7 @@ def _base_to_provider_dict(current_dict, include_project_id=False):
 
 # Note: The provider dict returned from this method will have provider
 #       data model objects in it.
-def lb_dict_to_provider_dict(lb_dict, vip=None, db_pools=None,
+def lb_dict_to_provider_dict(lb_dict, vip=None, add_vips=None, db_pools=None,
                              db_listeners=None, for_delete=False):
     new_lb_dict = _base_to_provider_dict(lb_dict, include_project_id=True)
     new_lb_dict['loadbalancer_id'] = new_lb_dict.pop('id')
@@ -139,6 +134,9 @@ def lb_dict_to_provider_dict(lb_dict, vip=None, db_pools=None,
         flavor_repo = repositories.FlavorRepository()
         new_lb_dict['flavor'] = flavor_repo.get_flavor_metadata_dict(
             db_api.get_session(), lb_dict['flavor_id'])
+    if add_vips:
+        new_lb_dict['additional_vips'] = db_additional_vips_to_provider_vips(
+            add_vips)
     if db_pools:
         new_lb_dict['pools'] = db_pools_to_provider_pools(
             db_pools, for_delete=for_delete)
@@ -173,6 +171,13 @@ def db_listeners_to_provider_listeners(db_listeners, for_delete=False):
     return provider_listeners
 
 
+def db_listeners_to_provider_dicts_list_of_dicts(db_listeners,
+                                                 for_delete=False):
+    listeners = db_listeners_to_provider_listeners(
+        db_listeners, for_delete=for_delete)
+    return [listener.to_dict() for listener in listeners]
+
+
 def db_listener_to_provider_listener(db_listener, for_delete=False):
     new_listener_dict = listener_dict_to_provider_dict(
         db_listener.to_dict(recurse=True), for_delete=for_delete)
@@ -204,6 +209,10 @@ def _get_secret_data(cert_manager, project_id, secret_ref, for_delete=False):
             secret_data = None
         else:
             raise exceptions.CertificateRetrievalException(ref=secret_ref)
+    # We need to have json convertible data for storing it in
+    # persistence jobboard backend.
+    if isinstance(secret_data, bytes):
+        return secret_data.decode()
     return secret_data
 
 
@@ -219,19 +228,6 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
     if 'tls_certificate_id' in new_listener_dict:
         new_listener_dict['default_tls_container_ref'] = new_listener_dict.pop(
             'tls_certificate_id')
-    if 'sni_containers' in new_listener_dict:
-        sni_refs = []
-        sni_containers = new_listener_dict.pop('sni_containers')
-        for sni in sni_containers:
-            if 'tls_container_id' in sni:
-                sni_refs.append(sni['tls_container_id'])
-            else:
-                raise exceptions.ValidationException(
-                    detail=_('Invalid SNI container on listener'))
-        new_listener_dict['sni_container_refs'] = sni_refs
-    if 'sni_container_refs' in listener_dict:
-        listener_dict['sni_containers'] = listener_dict.pop(
-            'sni_container_refs')
     if 'client_ca_tls_certificate_id' in new_listener_dict:
         new_listener_dict['client_ca_tls_container_ref'] = (
             new_listener_dict.pop('client_ca_tls_certificate_id'))
@@ -244,11 +240,9 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
         SNI_objs = []
         for sni in listener_obj.sni_containers:
             if isinstance(sni, dict):
-                if 'listener' in sni:
-                    del sni['listener']
                 sni_obj = data_models.SNI(**sni)
                 SNI_objs.append(sni_obj)
-            elif isinstance(sni, six.string_types):
+            elif isinstance(sni, str):
                 sni_obj = data_models.SNI(tls_container_id=sni)
                 SNI_objs.append(sni_obj)
             else:
@@ -281,11 +275,13 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
 
         if listener_obj.client_ca_tls_certificate_id:
             cert = _get_secret_data(cert_manager, listener_obj.project_id,
-                                    listener_obj.client_ca_tls_certificate_id)
+                                    listener_obj.client_ca_tls_certificate_id,
+                                    for_delete=for_delete)
             new_listener_dict['client_ca_tls_container_data'] = cert
         if listener_obj.client_crl_container_id:
             crl_file = _get_secret_data(cert_manager, listener_obj.project_id,
-                                        listener_obj.client_crl_container_id)
+                                        listener_obj.client_crl_container_id,
+                                        for_delete=for_delete)
             new_listener_dict['client_crl_container_data'] = crl_file
 
     # Format the allowed_cidrs
@@ -296,6 +292,20 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
         new_listener_dict['allowed_cidrs'] = [cidr_dict['cidr'] for
                                               cidr_dict in cidrs_dict_list]
 
+    # Format the sni_containers -> sni_container_refs
+    sni_containers = new_listener_dict.pop('sni_containers', None)
+    if sni_containers:
+        new_listener_dict['sni_container_refs'] = []
+        for sni in sni_containers:
+            if isinstance(sni, dict):
+                new_listener_dict['sni_container_refs'].append(
+                    sni['tls_container_id'])
+            elif isinstance(sni, str):
+                new_listener_dict['sni_container_refs'].append(sni)
+            else:
+                raise exceptions.ValidationException(
+                    detail=_('Invalid SNI container on listener'))
+
     # Remove the DB back references
     if 'load_balancer' in new_listener_dict:
         del new_listener_dict['load_balancer']
@@ -321,6 +331,14 @@ def listener_dict_to_provider_dict(listener_dict, for_delete=False):
     return new_listener_dict
 
 
+def db_additional_vips_to_provider_vips(db_add_vips):
+    provider_add_vips = []
+    for add_vip in db_add_vips:
+        provider_add_vips.append(
+            additional_vip_dict_to_provider_dict(add_vip.to_dict()))
+    return provider_add_vips
+
+
 def db_pools_to_provider_pools(db_pools, for_delete=False):
     provider_pools = []
     for pool in db_pools:
@@ -389,12 +407,14 @@ def pool_dict_to_provider_dict(pool_dict, for_delete=False):
 
         if pool_obj.ca_tls_certificate_id:
             cert = _get_secret_data(cert_manager, pool_obj.project_id,
-                                    pool_obj.ca_tls_certificate_id)
+                                    pool_obj.ca_tls_certificate_id,
+                                    for_delete=for_delete)
             new_pool_dict['ca_tls_container_data'] = cert
 
         if pool_obj.crl_container_id:
             crl_file = _get_secret_data(cert_manager, pool_obj.project_id,
-                                        pool_obj.crl_container_id)
+                                        pool_obj.crl_container_id,
+                                        for_delete=for_delete)
             new_pool_dict['crl_container_data'] = crl_file
 
     # Remove the DB back references
@@ -543,6 +563,22 @@ def vip_dict_to_provider_dict(vip_dict):
         new_vip_dict['vip_subnet_id'] = vip_dict['subnet_id']
     if 'qos_policy_id' in vip_dict:
         new_vip_dict['vip_qos_policy_id'] = vip_dict['qos_policy_id']
+    if constants.OCTAVIA_OWNED in vip_dict:
+        new_vip_dict[constants.OCTAVIA_OWNED] = vip_dict[
+            constants.OCTAVIA_OWNED]
+    return new_vip_dict
+
+
+def additional_vip_dict_to_provider_dict(vip_dict):
+    new_vip_dict = {}
+    if 'ip_address' in vip_dict:
+        new_vip_dict['ip_address'] = vip_dict['ip_address']
+    if 'network_id' in vip_dict:
+        new_vip_dict['network_id'] = vip_dict['network_id']
+    if 'port_id' in vip_dict:
+        new_vip_dict['port_id'] = vip_dict['port_id']
+    if 'subnet_id' in vip_dict:
+        new_vip_dict['subnet_id'] = vip_dict['subnet_id']
     return new_vip_dict
 
 
@@ -558,4 +594,19 @@ def provider_vip_dict_to_vip_obj(vip_dictionary):
         vip_obj.subnet_id = vip_dictionary['vip_subnet_id']
     if 'vip_qos_policy_id' in vip_dictionary:
         vip_obj.qos_policy_id = vip_dictionary['vip_qos_policy_id']
+    if constants.OCTAVIA_OWNED in vip_dictionary:
+        vip_obj.octavia_owned = vip_dictionary[constants.OCTAVIA_OWNED]
+    return vip_obj
+
+
+def provider_additional_vip_dict_to_additional_vip_obj(vip_dictionary):
+    vip_obj = data_models.AdditionalVip()
+    if 'ip_address' in vip_dictionary:
+        vip_obj.ip_address = vip_dictionary['ip_address']
+    if 'network_id' in vip_dictionary:
+        vip_obj.network_id = vip_dictionary['network_id']
+    if 'port_id' in vip_dictionary:
+        vip_obj.port_id = vip_dictionary['port_id']
+    if 'subnet_id' in vip_dictionary:
+        vip_obj.subnet_id = vip_dictionary['subnet_id']
     return vip_obj
