@@ -33,8 +33,11 @@ class SessionPersistencePOST(types.BaseType):
     """Defines mandatory and optional attributes of a POST request."""
     type = wtypes.wsattr(wtypes.Enum(str, *constants.SUPPORTED_SP_TYPES),
                          mandatory=True)
-    cookie_name = wtypes.wsattr(wtypes.StringType(max_length=255),
-                                default=None)
+    # pattern of invalid characters is based on
+    # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie
+    cookie_name = wtypes.wsattr(wtypes.StringType(
+        max_length=255, pattern=r'^[^\s,;\\]+$'),
+        default=None)
     persistence_timeout = wtypes.wsattr(wtypes.IntegerType(), default=None)
     persistence_granularity = wtypes.wsattr(types.IPAddressType(),
                                             default=None)
@@ -43,8 +46,11 @@ class SessionPersistencePOST(types.BaseType):
 class SessionPersistencePUT(types.BaseType):
     """Defines attributes that are acceptable of a PUT request."""
     type = wtypes.wsattr(wtypes.Enum(str, *constants.SUPPORTED_SP_TYPES))
-    cookie_name = wtypes.wsattr(wtypes.StringType(max_length=255),
-                                default=None)
+    # pattern of invalid characters is based on
+    # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie
+    cookie_name = wtypes.wsattr(wtypes.StringType(
+        max_length=255, pattern=r'^[^\s,;\\]+$'),
+        default=None)
     persistence_timeout = wtypes.wsattr(wtypes.IntegerType(), default=None)
     persistence_granularity = wtypes.wsattr(types.IPAddressType(),
                                             default=None)
@@ -86,6 +92,7 @@ class PoolResponse(BasePoolType):
     tls_enabled = wtypes.wsattr(bool)
     tls_ciphers = wtypes.wsattr(wtypes.StringType())
     tls_versions = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType()))
+    alpn_protocols = wtypes.wsattr(wtypes.ArrayType(types.AlpnProtocolType()))
 
     @classmethod
     def from_data_model(cls, data_model, children=False):
@@ -118,6 +125,7 @@ class PoolResponse(BasePoolType):
             member_model.from_data_model(i) for i in data_model.members]
 
         pool.tls_versions = data_model.tls_versions
+        pool.alpn_protocols = data_model.alpn_protocols
 
         return pool
 
@@ -167,6 +175,7 @@ class PoolPOST(BasePoolType):
     tls_ciphers = wtypes.wsattr(wtypes.StringType(max_length=2048))
     tls_versions = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(
         max_length=32)))
+    alpn_protocols = wtypes.wsattr(wtypes.ArrayType(types.AlpnProtocolType()))
 
 
 class PoolRootPOST(types.BaseType):
@@ -189,6 +198,7 @@ class PoolPUT(BasePoolType):
     tls_ciphers = wtypes.wsattr(wtypes.StringType(max_length=2048))
     tls_versions = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(
         max_length=32)))
+    alpn_protocols = wtypes.wsattr(wtypes.ArrayType(types.AlpnProtocolType()))
 
 
 class PoolRootPut(types.BaseType):
@@ -215,6 +225,7 @@ class PoolSingleCreate(BasePoolType):
     tls_ciphers = wtypes.wsattr(wtypes.StringType(max_length=2048))
     tls_versions = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(
         max_length=32)))
+    alpn_protocols = wtypes.wsattr(wtypes.ArrayType(types.AlpnProtocolType()))
 
 
 class PoolStatusResponse(BasePoolType):
