@@ -18,7 +18,6 @@ Routines for configuring Octavia
 """
 
 import os
-import ssl
 import sys
 
 from keystoneauth1 import loading as ks_loading
@@ -42,9 +41,6 @@ EXTRA_LOG_LEVEL_DEFAULTS = [
     'neutronclient.v2_0.client=INFO',
 ]
 
-TLS_PROTOCOL_CHOICES = [
-    p[9:].replace('_', '.') for p in ssl._PROTOCOL_NAMES.values()]
-
 core_opts = [
     cfg.HostnameOpt('host', default=utils.get_hostname(),
                     sample_default='<server-hostname.example.com>',
@@ -84,14 +80,13 @@ api_opts = [
                 help=_("Allow users to create TLS Terminated listeners?")),
     cfg.BoolOpt('allow_ping_health_monitors', default=True,
                 help=_("Allow users to create PING type Health Monitors?")),
+    cfg.BoolOpt('allow_prometheus_listeners', default=True,
+                help=_("Allow users to create PROMETHEUS type listeners?")),
     cfg.DictOpt('enabled_provider_drivers',
                 help=_('A comma separated list of dictionaries of the '
                        'enabled provider driver names and descriptions. '
                        'Must match the driver name in the '
-                       'octavia.api.drivers entrypoint. Example: '
-                       'amphora:The Octavia Amphora driver.,'
-                       'octavia:Deprecated alias of the Octavia '
-                       'Amphora driver.'),
+                       'octavia.api.drivers entrypoint.'),
                 default={'amphora': 'The Octavia Amphora driver.',
                          'octavia': 'Deprecated alias of the Octavia Amphora '
                          'driver.',
@@ -168,10 +163,10 @@ amphora_agent_opts = [
     cfg.IntOpt('agent_request_read_timeout', default=180,
                help=_("The time in seconds to allow a request from the "
                       "controller to run before terminating the socket.")),
-    cfg.StrOpt('agent_tls_protocol', default='TLSv1.2',
+    cfg.StrOpt('agent_tls_protocol', default=lib_consts.TLS_VERSION_1_2,
                help=_("Minimum TLS protocol for communication with the "
                       "amphora agent."),
-               choices=TLS_PROTOCOL_CHOICES),
+               choices=constants.TLS_ALL_VERSIONS),
 
     # Logging setup
     cfg.ListOpt('admin_log_targets',
@@ -309,6 +304,11 @@ health_manager_opts = [
                help=_('Sleep time between health checks in seconds.')),
     cfg.IntOpt('sock_rlimit', default=0,
                help=_(' sets the value of the heartbeat recv buffer')),
+    cfg.IntOpt('failover_threshold', default=None,
+               help=_('Stop failovers if the count of simultaneously failed '
+                      'amphora reaches this number. This may prevent large '
+                      'scale accidental failover events, like in the case of '
+                      'network failures or read-only database issues.')),
 
     # Used by the health manager on the amphora
     cfg.ListOpt('controller_ip_port_list',
@@ -354,8 +354,9 @@ haproxy_amphora_opts = [
     cfg.IntOpt('active_connection_max_retries',
                default=15,
                help=_('Retry threshold for connecting to active amphorae.')),
-    cfg.IntOpt('active_connection_rety_interval',
+    cfg.IntOpt('active_connection_retry_interval',
                default=2,
+               deprecated_name='active_connection_rety_interval',
                help=_('Retry timeout between connection attempts in '
                       'seconds for active amphora.')),
     cfg.IntOpt('failover_connection_max_retries',
@@ -380,7 +381,7 @@ haproxy_amphora_opts = [
                       'seconds.')),
     cfg.StrOpt('haproxy_stick_size', default='10k',
                help=_('Size of the HAProxy stick table. Accepts k, m, g '
-                      'suffixes.  Example: 10k')),
+                      'suffixes.')),
     cfg.StrOpt('user_log_format',
                default='{{ project_id }} {{ lb_id }} %f %ci %cp %t %{+Q}r %ST '
                        '%B %U %[ssl_c_verify] %{+Q}[ssl_c_s_dn] %b %s %Tt '
@@ -474,6 +475,10 @@ controller_worker_opts = [
                default='',
                help=_('Optional SSH keypair name, in nova, that will be used '
                       'for the authorized_keys inside the amphora.')),
+    cfg.StrOpt('amp_timezone',
+               default='UTC',
+               help=_('The timezone to use in the Amphora as represented in '
+                      '/usr/share/zoneinfo.')),
     cfg.ListOpt('amp_boot_network_list',
                 default='',
                 help=_('List of networks to attach to the Amphorae. '
@@ -526,16 +531,14 @@ controller_worker_opts = [
                help=_('Number of times an amphora delete should be retried.')),
     cfg.IntOpt('amphora_delete_retry_interval', default=5,
                help=_('Time, in seconds, between amphora delete retries.')),
+    cfg.BoolOpt('event_notifications', default=True),
 ]
 
 task_flow_opts = [
     cfg.StrOpt('engine',
                default='parallel',
                choices=constants.SUPPORTED_TASKFLOW_ENGINE_TYPES,
-               help=_('TaskFlow engine to use. '
-                      'serial - Runs all tasks on a single thread. '
-                      'parallel - Schedules tasks onto different threads to '
-                      'allow for running non-dependent tasks simultaneously')),
+               help=_('TaskFlow engine to use.')),
     cfg.IntOpt('max_workers',
                default=5,
                help=_('The maximum number of workers')),
@@ -552,7 +555,11 @@ task_flow_opts = [
                 help=_('If True, enables TaskFlow jobboard.')),
     cfg.StrOpt('jobboard_backend_driver',
                default='redis_taskflow_driver',
-               choices=['redis_taskflow_driver', 'zookeeper_taskflow_driver'],
+               choices=[('redis_taskflow_driver',
+                         'Driver that will use Redis to store job states.'),
+                        ('zookeeper_taskflow_driver',
+                         'Driver that will use Zookeeper to store job states.')
+                        ],
                help='Jobboard backend driver that will monitor job state.'),
     cfg.ListOpt('jobboard_backend_hosts', default=['127.0.0.1'],
                 help='Jobboard backend server host(s).'),
