@@ -150,6 +150,8 @@ PROTOCOL_TERMINATED_HTTPS = lib_consts.PROTOCOL_TERMINATED_HTTPS
 # 'PROXY'
 PROTOCOL_PROXY = lib_consts.PROTOCOL_PROXY
 SUPPORTED_PROTOCOLS = lib_consts.SUPPORTED_PROTOCOLS
+# 'PROMETHEUS'
+PROTOCOL_PROMETHEUS = lib_consts.PROTOCOL_PROMETHEUS
 
 # 'provisioning_status'
 PROVISIONING_STATUS = lib_consts.PROVISIONING_STATUS
@@ -159,6 +161,8 @@ AMPHORA_ALLOCATED = lib_consts.AMPHORA_ALLOCATED
 AMPHORA_BOOTING = lib_consts.AMPHORA_BOOTING
 # Amphora is ready to be allocated to a load balancer 'READY'
 AMPHORA_READY = lib_consts.AMPHORA_READY
+# 'FAILOVER_STOPPED'. Failover threshold level has been reached.
+AMPHORA_FAILOVER_STOPPED = lib_consts.AMPHORA_FAILOVER_STOPPED
 # 'ACTIVE'
 ACTIVE = lib_consts.ACTIVE
 # 'PENDING_DELETE'
@@ -215,7 +219,8 @@ VALID_LISTENER_POOL_PROTOCOL_MAP = {
     PROTOCOL_TERMINATED_HTTPS: [PROTOCOL_HTTP, PROTOCOL_PROXY,
                                 lib_consts.PROTOCOL_PROXYV2],
     PROTOCOL_UDP: [PROTOCOL_UDP],
-    lib_consts.PROTOCOL_SCTP: [lib_consts.PROTOCOL_SCTP]}
+    lib_consts.PROTOCOL_SCTP: [lib_consts.PROTOCOL_SCTP],
+    lib_consts.PROTOCOL_PROMETHEUS: []}
 
 # API Integer Ranges
 MIN_PORT_NUMBER = 1
@@ -232,8 +237,8 @@ DEFAULT_MAX_RETRIES_DOWN = 3
 MIN_HM_RETRIES = 1
 MAX_HM_RETRIES = 10
 
-# 1 year:     y     d    h    m    ms
-MAX_TIMEOUT = 365 * 24 * 60 * 60 * 1000
+# 24 days:   days  d    h    m    ms
+MAX_TIMEOUT = 24 * 24 * 60 * 60 * 1000
 MIN_TIMEOUT = 0
 
 DEFAULT_TIMEOUT_CLIENT_DATA = 50000
@@ -245,13 +250,6 @@ MUTABLE_STATUSES = (lib_consts.ACTIVE,)
 DELETABLE_STATUSES = (lib_consts.ACTIVE, lib_consts.ERROR)
 FAILOVERABLE_STATUSES = (lib_consts.ACTIVE, lib_consts.ERROR)
 
-# Note: The database Amphora table has a foreign key constraint against
-#       the provisioning_status table
-SUPPORTED_AMPHORA_STATUSES = (
-    lib_consts.AMPHORA_ALLOCATED, lib_consts.AMPHORA_BOOTING, lib_consts.ERROR,
-    lib_consts.AMPHORA_READY, lib_consts.DELETED, lib_consts.PENDING_CREATE,
-    lib_consts.PENDING_DELETE)
-
 AMPHORA_VM = 'VM'
 SUPPORTED_AMPHORA_TYPES = (AMPHORA_VM,)
 
@@ -293,12 +291,16 @@ DOMAIN_NAME_REGEX = (
     r'^(?=.{1,253}\.?$)(?:(?!-|[^.]+_)[A-Za-z0-9-_]{1,63}(?<!-)(?:\.|$))+$')
 
 # TaskFlow
-SUPPORTED_TASKFLOW_ENGINE_TYPES = ['serial', 'parallel']
+SUPPORTED_TASKFLOW_ENGINE_TYPES = [
+    ('serial', 'Runs all tasks on a single thread'),
+    ('parallel', 'Schedules tasks onto different threads to allow for running '
+                 'non-dependent tasks simultaneously')]
 
 # Task/Flow constants
 ACTIVE_CONNECTIONS = 'active_connections'
 ADD_NICS = 'add_nics'
-ADDED_PORTS = 'added_ports'
+ADD_SUBNETS = 'add_subnets'
+ADDITIONAL_VIPS = 'additional_vips'
 ADMIN_STATE_UP = 'admin_state_up'
 ALLOWED_ADDRESS_PAIRS = 'allowed_address_pairs'
 AMP_DATA = 'amp_data'
@@ -330,9 +332,11 @@ CREATED_AT = 'created_at'
 CRL_CONTAINER_ID = 'crl_container_id'
 DEFAULT_TLS_CONTAINER_DATA = 'default_tls_container_data'
 DELETE_NICS = 'delete_nics'
+DELETE_SUBNETS = 'delete_subnets'
 DELTA = 'delta'
 DELTAS = 'deltas'
 DESCRIPTION = 'description'
+DESTINATION = 'destination'
 DEVICE_OWNER = 'device_owner'
 ENABLED = 'enabled'
 FAILED_AMP_VRRP_PORT_ID = 'failed_amp_vrrp_port_id'
@@ -342,6 +346,7 @@ FAILOVER_AMPHORA_ID = 'failover_amphora_id'
 FIELDS = 'fields'
 FIXED_IPS = 'fixed_ips'
 FLAVOR_ID = 'flavor_id'
+GATEWAY_IP = 'gateway_ip'
 HA_IP = 'ha_ip'
 HA_PORT_ID = 'ha_port_id'
 HEALTH_MON = 'health_mon'
@@ -349,6 +354,7 @@ HEALTH_MONITOR = 'health_monitor'
 HEALTH_MONITOR_ID = 'health_monitor_id'
 HEALTHMONITOR_ID = 'healthmonitor_id'
 HEALTH_MONITOR_UPDATES = 'health_monitor_updates'
+HOST_ROUTES = 'host_routes'
 ID = 'id'
 IMAGE_ID = 'image_id'
 IP_ADDRESS = 'ip_address'
@@ -367,6 +373,7 @@ LOADBALANCER = 'loadbalancer'
 LOADBALANCER_ID = 'loadbalancer_id'
 LOAD_BALANCER_ID = 'load_balancer_id'
 LOAD_BALANCER_UPDATES = 'load_balancer_updates'
+MAC_ADDRESS = 'mac_address'
 MANAGEMENT_NETWORK = 'management_network'
 MEMBER = 'member'
 MEMBER_ID = 'member_id'
@@ -376,6 +383,7 @@ MESSAGE = 'message'
 NAME = 'name'
 NETWORK = 'network'
 NETWORK_ID = 'network_id'
+NEXTHOP = 'nexthop'
 NICS = 'nics'
 OBJECT = 'object'
 ORIGINAL_HEALTH_MONITOR = 'original_health_monitor'
@@ -423,10 +431,12 @@ TOPOLOGY = 'topology'
 TOTAL_CONNECTIONS = 'total_connections'
 UPDATED_AT = 'updated_at'
 UPDATE_DICT = 'update_dict'
+UPDATED_PORTS = 'updated_ports'
 VALID_VIP_NETWORKS = 'valid_vip_networks'
 VIP = 'vip'
 VIP_ADDRESS = 'vip_address'
 VIP_NETWORK = 'vip_network'
+VIP_NETWORK_ID = 'vip_network_id'
 VIP_PORT_ID = 'vip_port_id'
 VIP_QOS_POLICY_ID = 'vip_qos_policy_id'
 VIP_SG_ID = 'vip_sg_id'
@@ -496,6 +506,7 @@ GENERATE_SERVER_PEM = 'octavia-generate-serverpem'
 UPDATE_CERT_EXPIRATION = 'octavia-update-cert-expiration'
 CERT_COMPUTE_CREATE = 'octavia-cert-compute-create'
 COMPUTE_CREATE = 'octavia-compute-create'
+COMPUTE_CREATE_RETRY_SUBFLOW = 'octavia-compute-create-retry-subflow'
 UPDATE_AMPHORA_COMPUTEID = 'octavia-update-amphora-computeid'
 MARK_AMPHORA_BOOTING_INDB = 'octavia-mark-amphora-booting-indb'
 WAIT_FOR_AMPHORA = 'octavia-wait_for_amphora'
@@ -579,7 +590,6 @@ RPC_NAMESPACE_CONTROLLER_AGENT = 'controller'
 # Build Type Priority
 LB_CREATE_FAILOVER_PRIORITY = 20
 LB_CREATE_NORMAL_PRIORITY = 40
-LB_CREATE_SPARES_POOL_PRIORITY = 60
 LB_CREATE_ADMIN_FAILOVER_PRIORITY = 80
 BUILD_TYPE_PRIORITY = 'build_type_priority'
 
@@ -714,8 +724,6 @@ NOAUTH = 'noauth'
 TESTING = 'testing'
 
 # Amphora distro-specific data
-UBUNTU_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/network/interfaces.d/'
-RH_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/sysconfig/network-scripts/'
 UBUNTU = 'ubuntu'
 CENTOS = 'centos'
 
@@ -731,6 +739,7 @@ DEFAULT_PAGE_SIZE = 1000
 
 # RBAC
 LOADBALANCER_API = 'os_load-balancer_api'
+
 RULE_API_ADMIN = 'rule:load-balancer:admin'
 RULE_API_READ = 'rule:load-balancer:read'
 RULE_API_READ_GLOBAL = 'rule:load-balancer:read-global'
@@ -738,6 +747,7 @@ RULE_API_WRITE = 'rule:load-balancer:write'
 RULE_API_READ_QUOTA = 'rule:load-balancer:read-quota'
 RULE_API_READ_QUOTA_GLOBAL = 'rule:load-balancer:read-quota-global'
 RULE_API_WRITE_QUOTA = 'rule:load-balancer:write-quota'
+
 RBAC_LOADBALANCER = '{}:loadbalancer:'.format(LOADBALANCER_API)
 RBAC_LISTENER = '{}:listener:'.format(LOADBALANCER_API)
 RBAC_POOL = '{}:pool:'.format(LOADBALANCER_API)
@@ -756,6 +766,7 @@ RBAC_FLAVOR_PROFILE = '{}:flavor-profile:'.format(LOADBALANCER_API)
 RBAC_AVAILABILITY_ZONE = '{}:availability-zone:'.format(LOADBALANCER_API)
 RBAC_AVAILABILITY_ZONE_PROFILE = '{}:availability-zone-profile:'.format(
     LOADBALANCER_API)
+
 RBAC_POST = 'post'
 RBAC_PUT = 'put'
 RBAC_PUT_CONFIG = 'put_config'
@@ -768,9 +779,21 @@ RBAC_GET_DEFAULTS = 'get_defaults'
 RBAC_GET_STATS = 'get_stats'
 RBAC_GET_STATUS = 'get_status'
 
+RBAC_SCOPE_PROJECT = 'project'
+RBAC_SCOPE_SYSTEM = 'system'
+
+RBAC_ROLES_DEPRECATED_REASON = (
+    'The Octavia API now requires the OpenStack default roles and scoped '
+    'tokens. '
+    'See https://docs.openstack.org/octavia/latest/configuration/policy.html '
+    'and https://docs.openstack.org/keystone/latest/contributor/'
+    'services.html#reusable-default-roles for more information.')
+
 # PROVIDERS
 OCTAVIA = 'octavia'
 AMPHORAV2 = 'amphorav2'
+# Deprecated in Z, to be removed
+AMPHORAV1 = 'amphorav1'
 
 # systemctl commands
 DISABLE = 'disable'
@@ -781,6 +804,8 @@ AMP_NETNS_SVC_PREFIX = 'amphora-netns'
 
 # Amphora Feature Compatibility
 HTTP_REUSE = 'has_http_reuse'
+POOL_ALPN = 'has_pool_alpn'
+INSECURE_FORK = 'requires_insecure_fork'
 
 # TODO(johnsom) convert these to octavia_lib constants
 # once octavia is transitioned to use octavia_lib
@@ -807,6 +832,12 @@ TOPIC_AMPHORA_V2 = 'octavia_provisioning_v2'
 HAPROXY_HTTP_PROTOCOLS = [lib_consts.PROTOCOL_HTTP,
                           lib_consts.PROTOCOL_TERMINATED_HTTPS]
 
+LVS_PROTOCOLS = [PROTOCOL_UDP,
+                 lib_consts.PROTOCOL_SCTP]
+
+HAPROXY_BACKEND = 'HAPROXY'
+LVS_BACKEND = 'LVS'
+
 # Map each supported protocol to its L4 protocol
 L4_PROTOCOL_MAP = {
     PROTOCOL_TCP: PROTOCOL_TCP,
@@ -817,6 +848,7 @@ L4_PROTOCOL_MAP = {
     lib_consts.PROTOCOL_PROXYV2: PROTOCOL_TCP,
     PROTOCOL_UDP: PROTOCOL_UDP,
     lib_consts.PROTOCOL_SCTP: lib_consts.PROTOCOL_SCTP,
+    lib_consts.PROTOCOL_PROMETHEUS: lib_consts.PROTOCOL_TCP,
 }
 
 # Image drivers
@@ -874,3 +906,41 @@ SUPPORTED_ALPN_PROTOCOLS = [lib_consts.ALPN_PROTOCOL_HTTP_2,
 AMPHORA_SUPPORTED_ALPN_PROTOCOLS = [lib_consts.ALPN_PROTOCOL_HTTP_2,
                                     lib_consts.ALPN_PROTOCOL_HTTP_1_1,
                                     lib_consts.ALPN_PROTOCOL_HTTP_1_0]
+
+# Amphora interface fields
+MTU = 'mtu'
+ADDRESSES = 'addresses'
+ROUTES = 'routes'
+RULES = 'rules'
+SCRIPTS = 'scripts'
+
+# pyroute2 fields
+STATE = 'state'
+
+FAMILY = 'family'
+ADDRESS = 'address'
+PREFIXLEN = 'prefixlen'
+DHCP = 'dhcp'
+IPV6AUTO = 'ipv6auto'
+
+DST = 'dst'
+PREFSRC = 'prefsrc'
+GATEWAY = 'gateway'
+FLAGS = 'flags'
+ONLINK = 'onlink'
+TABLE = 'table'
+SCOPE = 'scope'
+
+SRC = 'src'
+SRC_LEN = 'src_len'
+
+IFACE_UP = 'up'
+IFACE_DOWN = 'down'
+
+COMMAND = 'command'
+
+IFLA_ADDRESS = 'IFLA_ADDRESS'
+IFLA_IFNAME = 'IFLA_IFNAME'
+
+# Amphora network directory
+AMP_NET_DIR_TEMPLATE = '/etc/octavia/interfaces/'
