@@ -26,62 +26,68 @@ LOG = logging.getLogger(__name__)
 
 
 class BaseDataModel(object):
+    def _to_dict(self, value, calling_classes=None, recurse=False):
+        calling_classes = calling_classes or []
+        # We need to have json convertible data for storing it in
+        # persistence jobboard backend.
+        if isinstance(value, datetime.datetime):
+            ret = value.isoformat()
+        elif isinstance(value, bytes):
+            ret = value.decode()
+        elif recurse:
+            if isinstance(value, list):
+                ret = []
+                for item in value:
+                    if isinstance(item, BaseDataModel):
+                        if type(self) not in calling_classes:
+                            ret.append(
+                                item.to_dict(calling_classes=(
+                                    calling_classes + [type(self)]),
+                                    recurse=recurse))
+                        else:
+                            # TODO(rm_work): Is the idea that if this list
+                            #  contains ANY BaseDataModel, that all of them
+                            #  are data models, and we may as well quit?
+                            #  Or, were we supposed to append a `None` for
+                            #  each one? I assume the former?
+                            ret = None
+                            break
+                    else:
+                        ret.append(
+                            self._to_dict(item,
+                                          calling_classes=calling_classes,
+                                          recurse=recurse))
+            elif isinstance(value, BaseDataModel):
+                if type(self) not in calling_classes:
+                    ret = value.to_dict(
+                        calling_classes=calling_classes + [type(self)],
+                        recurse=recurse)
+                else:
+                    ret = None
+            else:
+                ret = value
+        else:
+            if isinstance(value, BaseDataModel):
+                ret = None
+            elif isinstance(value, list):
+                ret = []
+            else:
+                ret = value
+        return ret
+
     def to_dict(self, calling_classes=None, recurse=False, **kwargs):
         """Converts a data model to a dictionary."""
         calling_classes = calling_classes or []
         ret = {}
-        for attr in self.__dict__:
+        for attr, value in self.__dict__.items():
             if attr.startswith('_') or not kwargs.get(attr, True):
                 continue
-            value = self.__dict__[attr]
             if attr == 'tags':
                 # tags is a list, it doesn't need recurse
                 ret[attr] = value
                 continue
-            # We need to have json convertible data for storing it in
-            # persistence jobboard backend.
-            if isinstance(value, datetime.datetime):
-                ret[attr] = value.isoformat()
-                continue
-            if isinstance(value, bytes):
-                ret[attr] = value.decode()
-                continue
-            if recurse:
-                if isinstance(getattr(self, attr), list):
-                    ret[attr] = []
-                    for item in value:
-                        if isinstance(item, BaseDataModel):
-                            if type(self) not in calling_classes:
-                                ret[attr].append(
-                                    item.to_dict(calling_classes=(
-                                        calling_classes + [type(self)]),
-                                        recurse=recurse))
-                            else:
-                                # TODO(rm_work): Is the idea that if this list
-                                #  contains ANY BaseDataModel, that all of them
-                                #  are data models, and we may as well quit?
-                                #  Or, were we supposed to append a `None` for
-                                #  each one? I assume the former?
-                                ret[attr] = None
-                                break
-                        else:
-                            ret[attr].append(item)
-                elif isinstance(getattr(self, attr), BaseDataModel):
-                    if type(self) not in calling_classes:
-                        ret[attr] = value.to_dict(
-                            calling_classes=calling_classes + [type(self)],
-                            recurse=recurse)
-                    else:
-                        ret[attr] = None
-                else:
-                    ret[attr] = value
-            else:
-                if isinstance(getattr(self, attr), BaseDataModel):
-                    ret[attr] = None
-                elif isinstance(getattr(self, attr), list):
-                    ret[attr] = []
-                else:
-                    ret[attr] = value
+            ret[attr] = self._to_dict(value, calling_classes=calling_classes,
+                                      recurse=recurse)
 
         return ret
 
@@ -293,7 +299,8 @@ class Pool(BaseDataModel):
                  created_at=None, updated_at=None, provisioning_status=None,
                  tags=None, tls_certificate_id=None,
                  ca_tls_certificate_id=None, crl_container_id=None,
-                 tls_enabled=None, tls_ciphers=None, tls_versions=None):
+                 tls_enabled=None, tls_ciphers=None, tls_versions=None,
+                 alpn_protocols=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -319,6 +326,7 @@ class Pool(BaseDataModel):
         self.tls_enabled = tls_enabled
         self.tls_ciphers = tls_ciphers
         self.tls_versions = tls_versions
+        self.alpn_protocols = alpn_protocols
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -488,7 +496,8 @@ class LoadBalancer(BaseDataModel):
                  topology=None, vip=None, listeners=None, amphorae=None,
                  pools=None, vrrp_group=None, server_group_id=None,
                  created_at=None, updated_at=None, provider=None, tags=None,
-                 flavor_id=None, availability_zone=None):
+                 flavor_id=None, availability_zone=None,
+                 additional_vips=None):
 
         self.id = id
         self.project_id = project_id
@@ -510,6 +519,7 @@ class LoadBalancer(BaseDataModel):
         self.tags = tags or []
         self.flavor_id = flavor_id
         self.availability_zone = availability_zone
+        self.additional_vips = additional_vips or []
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -552,6 +562,18 @@ class Vip(BaseDataModel):
         self.octavia_owned = octavia_owned
 
 
+class AdditionalVip(BaseDataModel):
+
+    def __init__(self, load_balancer_id=None, ip_address=None, subnet_id=None,
+                 network_id=None, port_id=None, load_balancer=None):
+        self.load_balancer_id = load_balancer_id
+        self.ip_address = ip_address
+        self.subnet_id = subnet_id
+        self.network_id = network_id
+        self.port_id = port_id
+        self.load_balancer = load_balancer
+
+
 class SNI(BaseDataModel):
 
     def __init__(self, listener_id=None, position=None, listener=None,
