@@ -17,6 +17,7 @@ import re
 
 import jinja2
 from octavia_lib.common import constants as lib_consts
+from oslo_utils import versionutils
 
 from octavia.common.config import cfg
 from octavia.common import constants
@@ -28,7 +29,8 @@ PROTOCOL_MAP = {
     constants.PROTOCOL_HTTPS: 'tcp',
     constants.PROTOCOL_PROXY: 'proxy',
     lib_consts.PROTOCOL_PROXYV2: 'proxy',
-    constants.PROTOCOL_TERMINATED_HTTPS: 'http'
+    constants.PROTOCOL_TERMINATED_HTTPS: 'http',
+    lib_consts.PROTOCOL_PROMETHEUS: 'http'
 }
 
 BALANCE_MAP = {
@@ -97,12 +99,17 @@ class JinjaTemplater(object):
         # pair might be running an older amphora version.
 
         feature_compatibility = {}
+        version = ".".join(haproxy_versions)
         # Is it newer than haproxy 1.5?
-        if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 6):
+        if versionutils.is_compatible("1.6.0", version, same_major=False):
             feature_compatibility[constants.HTTP_REUSE] = True
-            feature_compatibility[constants.SERVER_STATE_FILE] = True
-        if not (int(haproxy_versions[0]) < 2 and int(haproxy_versions[1]) < 9):
+        if versionutils.is_compatible("1.9.0", version, same_major=False):
             feature_compatibility[constants.POOL_ALPN] = True
+        if int(haproxy_versions[0]) >= 2:
+            feature_compatibility[lib_consts.PROTOCOL_PROMETHEUS] = True
+        # haproxy 2.2 requires insecure-fork-wanted for PING healthchecks
+        if versionutils.is_compatible("2.2.0", version, same_major=False):
+            feature_compatibility[constants.INSECURE_FORK] = True
 
         return self.render_loadbalancer_obj(
             host_amphora, listeners, tls_certs=tls_certs,
@@ -165,8 +172,16 @@ class JinjaTemplater(object):
                                           listeners[0].load_balancer.id)
         state_file_path = '%s/%s/servers-state' % (
             self.base_amp_path,
-            listeners[0].load_balancer.id) if feature_compatibility.get(
-            constants.SERVER_STATE_FILE) else ''
+            listeners[0].load_balancer.id)
+        prometheus_listener = False
+        for listener in listeners:
+            if listener.protocol == lib_consts.PROTOCOL_PROMETHEUS:
+                prometheus_listener = True
+                break
+        require_insecure_fork = feature_compatibility.get(
+            constants.INSECURE_FORK)
+        enable_prometheus = prometheus_listener and feature_compatibility.get(
+            lib_consts.PROTOCOL_PROMETHEUS, False)
         return self._get_template().render(
             {'loadbalancer': loadbalancer,
              'stats_sock': socket_path,
@@ -176,7 +191,9 @@ class JinjaTemplater(object):
              'administrative_log_facility':
                  CONF.amphora_agent.administrative_log_facility,
              'user_log_facility': CONF.amphora_agent.user_log_facility,
-             'connection_logging': self.connection_logging},
+             'connection_logging': self.connection_logging,
+             'enable_prometheus': enable_prometheus,
+             'require_insecure_fork': require_insecure_fork},
             constants=constants, lib_consts=lib_consts)
 
     def _transform_loadbalancer(self, host_amphora, loadbalancer, listeners,
@@ -191,10 +208,13 @@ class JinjaTemplater(object):
                 continue
             listener_transforms.append(self._transform_listener(
                 listener, tls_certs, feature_compatibility, loadbalancer))
+        additional_vips = [
+            vip.ip_address for vip in loadbalancer.additional_vips]
 
         ret_value = {
             'id': loadbalancer.id,
             'vip_address': loadbalancer.vip.ip_address,
+            'additional_vips': additional_vips,
             'listeners': listener_transforms,
             'topology': loadbalancer.topology,
             'enabled': loadbalancer.enabled,
@@ -264,6 +284,8 @@ class JinjaTemplater(object):
                 CONF.haproxy_amphora.timeout_member_data),
             'timeout_tcp_inspect': (listener.timeout_tcp_inspect or
                                     CONF.haproxy_amphora.timeout_tcp_inspect),
+            lib_consts.PROTOCOL_PROMETHEUS: feature_compatibility.get(
+                lib_consts.PROTOCOL_PROMETHEUS, False)
         }
         if self.connection_logging:
             ret_value['user_log_format'] = (
@@ -294,7 +316,8 @@ class JinjaTemplater(object):
                                  tls_certs[listener.client_crl_container_id]))
 
         tls_enabled = False
-        if listener.protocol == constants.PROTOCOL_TERMINATED_HTTPS:
+        if listener.protocol in (constants.PROTOCOL_TERMINATED_HTTPS,
+                                 constants.PROTOCOL_PROMETHEUS):
             tls_enabled = True
             if listener.tls_ciphers is not None:
                 ret_value['tls_ciphers'] = listener.tls_ciphers
