@@ -22,9 +22,9 @@ Defined here so these can also be used at deeper levels than the API.
 import ipaddress
 import re
 
-import netaddr
 from oslo_config import cfg
-import rfc3986
+from rfc3986 import uri_reference
+from rfc3986 import validators
 from wsme import types as wtypes
 
 from octavia.common import constants
@@ -37,13 +37,16 @@ CONF = cfg.CONF
 
 def url(url, require_scheme=True):
     """Raises an error if the url doesn't look like a URL."""
+    validator = validators.Validator()
+    if require_scheme:
+        validator.allow_schemes('http', 'https')
+        validator.require_presence_of('scheme', 'host')
+        validator.check_validity_of('scheme', 'host', 'path')
+    else:
+        validator.check_validity_of('path')
+
     try:
-        if not rfc3986.is_valid_uri(url, require_scheme=require_scheme):
-            raise exceptions.InvalidURL(url=url)
-        p_url = rfc3986.urlparse(rfc3986.normalize_uri(url))
-        if require_scheme:
-            if p_url.scheme != 'http' and p_url.scheme != 'https':
-                raise exceptions.InvalidURL(url=url)
+        validator.validate(uri_reference(url))
     except Exception as e:
         raise exceptions.InvalidURL(url=url) from e
     return True
@@ -51,10 +54,13 @@ def url(url, require_scheme=True):
 
 def url_path(url_path):
     """Raises an error if the url_path doesn't look like a URL Path."""
+    validator = validators.Validator().check_validity_of('path')
     try:
-        p_url = rfc3986.urlparse(rfc3986.normalize_uri(url_path))
+        p_url = uri_reference(url_path)
+        validator.validate(p_url)
 
         invalid_path = (
+            re.search(r"\s", url_path) or
             p_url.scheme or p_url.userinfo or p_url.host or
             p_url.port or
             p_url.path is None or
@@ -390,7 +396,7 @@ def network_allowed_by_config(network_id, valid_networks=None):
 
 
 def is_ip_member_of_cidr(address, cidr):
-    if netaddr.IPAddress(address) in netaddr.IPNetwork(cidr):
+    if ipaddress.ip_address(address) in ipaddress.ip_network(cidr):
         return True
     return False
 
@@ -429,14 +435,6 @@ def ip_not_reserved(ip_address):
                                        option='member address')
 
 
-def is_flavor_spares_compatible(flavor):
-    if flavor:
-        # If a compute flavor is specified, the flavor is not spares compatible
-        if flavor.get(constants.COMPUTE_FLAVOR, None):
-            return False
-    return True
-
-
 def check_cipher_prohibit_list(cipherstring):
     ciphers = cipherstring.split(':')
     prohibit_list = CONF.api_settings.tls_cipher_prohibit_list.split(':')
