@@ -45,9 +45,8 @@ class LoadBalancerFlows(object):
         self.member_flows = member_flows.MemberFlows()
 
     def get_create_load_balancer_flow(self, topology, listeners=None):
-        """Creates a conditional graph flow that allocates a loadbalancer to
+        """Creates a conditional graph flow that allocates a loadbalancer.
 
-        two spare amphorae.
         :raises InvalidTopology: Invalid topology specified
         :return: The graph flow for creating a loadbalancer.
         """
@@ -203,12 +202,12 @@ class LoadBalancerFlows(object):
         )
         flows.append(
             network_tasks.HandleNetworkDeltas(
-                requires=constants.DELTAS, provides=constants.ADDED_PORTS
+                requires=constants.DELTAS, provides=constants.UPDATED_PORTS
             )
         )
         flows.append(
             amphora_driver_tasks.AmphoraePostNetworkPlug(
-                requires=(constants.LOADBALANCER, constants.ADDED_PORTS)
+                requires=(constants.LOADBALANCER, constants.UPDATED_PORTS)
             )
         )
         flows.append(
@@ -392,7 +391,7 @@ class LoadBalancerFlows(object):
             constants.CONN_MAX_RETRIES:
                 CONF.haproxy_amphora.active_connection_max_retries,
             constants.CONN_RETRY_INTERVAL:
-                CONF.haproxy_amphora.active_connection_rety_interval}
+                CONF.haproxy_amphora.active_connection_retry_interval}
 
         if failed_amp:
             if failed_amp.role in (constants.ROLE_MASTER,
@@ -400,8 +399,6 @@ class LoadBalancerFlows(object):
                 amp_role = 'master_or_backup'
             elif failed_amp.role == constants.ROLE_STANDALONE:
                 amp_role = 'standalone'
-            elif failed_amp.role is None:
-                amp_role = 'spare'
             else:
                 amp_role = 'undefined'
             LOG.info("Performing failover for amphora: %s",
@@ -421,8 +418,8 @@ class LoadBalancerFlows(object):
 
         # Check that the VIP port exists and is ok
         failover_LB_flow.add(
-            network_tasks.AllocateVIP(requires=constants.LOADBALANCER,
-                                      provides=constants.VIP))
+            network_tasks.AllocateVIPforFailover(
+                requires=constants.LOADBALANCER, provides=constants.VIP))
 
         # Update the database with the VIP information
         failover_LB_flow.add(database_tasks.UpdateVIPAfterAllocation(
@@ -547,8 +544,6 @@ class LoadBalancerFlows(object):
                     amp_role = 'master_or_backup'
                 elif failed_amp.role == constants.ROLE_STANDALONE:
                     amp_role = 'standalone'
-                elif failed_amp.role is None:
-                    amp_role = 'spare'
                 else:
                     amp_role = 'undefined'
                 LOG.info("Performing failover for amphora: %s",
