@@ -31,6 +31,7 @@ from octavia.controller.worker.v2.tasks import compute_tasks
 from octavia.controller.worker.v2.tasks import database_tasks
 from octavia.controller.worker.v2.tasks import lifecycle_tasks
 from octavia.controller.worker.v2.tasks import network_tasks
+from octavia.controller.worker.v2.tasks import notification_tasks
 from octavia.db import repositories as repo
 
 CONF = cfg.CONF
@@ -66,10 +67,13 @@ class LoadBalancerFlows(object):
         ))
         lb_create_flow.add(network_tasks.AllocateVIP(
             requires=constants.LOADBALANCER,
-            provides=constants.VIP))
+            provides=(constants.VIP, constants.ADDITIONAL_VIPS)))
         lb_create_flow.add(database_tasks.UpdateVIPAfterAllocation(
             requires=(constants.LOADBALANCER_ID, constants.VIP),
             provides=constants.LOADBALANCER))
+        lb_create_flow.add(database_tasks.UpdateAdditionalVIPsAfterAllocation(
+            requires=(constants.LOADBALANCER_ID, constants.ADDITIONAL_VIPS),
+            provides=constants.LOADBALANCER))
         lb_create_flow.add(network_tasks.UpdateVIPSecurityGroup(
             requires=constants.LOADBALANCER_ID))
         lb_create_flow.add(network_tasks.GetSubnetFromVIP(
@@ -93,6 +97,13 @@ class LoadBalancerFlows(object):
         if listeners:
             lb_create_flow.add(*self._create_listeners_flow())
 
+        if CONF.controller_worker.event_notifications:
+            lb_create_flow.add(
+                notification_tasks.SendCreateNotification(
+                    requires=constants.LOADBALANCER
+                )
+            )
+
         return lb_create_flow
 
     def _create_single_topology(self):
@@ -309,6 +320,9 @@ class LoadBalancerFlows(object):
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(database_tasks.DecrementLoadBalancerQuota(
             requires=constants.PROJECT_ID))
+        if CONF.controller_worker.event_notifications:
+            delete_LB_flow.add(notification_tasks.SendDeleteNotification(
+                requires=constants.LOADBALANCER))
         return delete_LB_flow
 
     def get_cascade_delete_load_balancer_flow(self, lb, listeners, pools):
@@ -336,6 +350,12 @@ class LoadBalancerFlows(object):
             requires=[constants.LOADBALANCER, constants.UPDATE_DICT]))
         update_LB_flow.add(database_tasks.MarkLBActiveInDB(
             requires=constants.LOADBALANCER))
+        if CONF.controller_worker.event_notifications:
+            update_LB_flow.add(
+                notification_tasks.SendUpdateNotification(
+                    requires=constants.LOADBALANCER
+                )
+            )
 
         return update_LB_flow
 
@@ -375,7 +395,7 @@ class LoadBalancerFlows(object):
             constants.CONN_MAX_RETRIES:
                 CONF.haproxy_amphora.active_connection_max_retries,
             constants.CONN_RETRY_INTERVAL:
-                CONF.haproxy_amphora.active_connection_rety_interval}
+                CONF.haproxy_amphora.active_connection_retry_interval}
 
         if failed_amp:
             failed_amp_role = failed_amp.get(constants.ROLE)
@@ -404,12 +424,18 @@ class LoadBalancerFlows(object):
         # Check that the VIP port exists and is ok
         failover_LB_flow.add(
             network_tasks.AllocateVIPforFailover(
-                requires=constants.LOADBALANCER, provides=constants.VIP))
+                requires=constants.LOADBALANCER,
+                provides=(constants.VIP, constants.ADDITIONAL_VIPS)))
 
         # Update the database with the VIP information
         failover_LB_flow.add(database_tasks.UpdateVIPAfterAllocation(
             requires=(constants.LOADBALANCER_ID, constants.VIP),
             provides=constants.LOADBALANCER))
+        failover_LB_flow.add(
+            database_tasks.UpdateAdditionalVIPsAfterAllocation(
+                requires=(constants.LOADBALANCER_ID,
+                          constants.ADDITIONAL_VIPS),
+                provides=constants.LOADBALANCER))
 
         # Make sure the SG has the correct rules and re-apply to the
         # VIP port. It is not used on the VIP port, but will help lock
