@@ -237,6 +237,15 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         primaryjoin='and_(foreign(Tags.resource_id)==Member.id)'
     )
 
+    def __str__(self):
+        return (f"Member(id={self.id!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, "
+                f"provisioning_status={self.provisioning_status!r}, "
+                f"ip_address={self.ip_address!r}, "
+                f"protocol_port={self.protocol_port!r}, "
+                f"operating_status={self.operating_status!r}, "
+                f"weight={self.weight!r})")
+
 
 class HealthMonitor(base_models.BASE, base_models.IdMixin,
                     base_models.ProjectMixin, models.TimestampMixin,
@@ -293,6 +302,11 @@ class HealthMonitor(base_models.BASE, base_models.IdMixin,
     http_version = sa.Column(sa.Float, nullable=True)
     domain_name = sa.Column(sa.String(255), nullable=True)
 
+    def __str__(self):
+        return (f"HealthMonitor(id={self.id!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, type={self.type!r}, "
+                f"enabled={self.enabled!r})")
+
 
 class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
            models.TimestampMixin, base_models.NameMixin, base_models.TagMixin):
@@ -353,6 +367,7 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     tls_enabled = sa.Column(sa.Boolean, default=False, nullable=False)
     tls_ciphers = sa.Column(sa.String(2048), nullable=True)
     tls_versions = sa.Column(ScalarListType(), nullable=True)
+    alpn_protocols = sa.Column(ScalarListType(), nullable=True)
 
     # This property should be a unique list of any listeners that reference
     # this pool as its default_pool and any listeners referenced by enabled
@@ -371,6 +386,14 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
                 _l_ids.append(li.id)
         return _listeners
 
+    def __str__(self):
+        return (f"Pool(id={self.id!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, "
+                f"provisioning_status={self.provisioning_status!r}, "
+                f"protocol={self.protocol!r}, "
+                f"lb_algorithm={self.lb_algorithm!r}, "
+                f"enabled={self.enabled!r})")
+
 
 class LoadBalancer(base_models.BASE, base_models.IdMixin,
                    base_models.ProjectMixin, models.TimestampMixin,
@@ -404,6 +427,9 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
     provider = sa.Column(sa.String(64), nullable=True)
     vip = orm.relationship('Vip', cascade='delete', uselist=False,
                            backref=orm.backref('load_balancer', uselist=False))
+    additional_vips = orm.relationship(
+        'AdditionalVip', cascade='delete', uselist=True,
+        backref=orm.backref('load_balancer', uselist=False))
     pools = orm.relationship('Pool', cascade='delete', uselist=True,
                              back_populates="load_balancer")
     listeners = orm.relationship('Listener', cascade='delete', uselist=True,
@@ -424,6 +450,13 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
                       name="fk_load_balancer_availability_zone_name"),
         nullable=True)
 
+    def __str__(self):
+        return (f"LoadBalancer(id={self.id!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, vip={self.vip!r}, "
+                f"provisioning_status={self.provisioning_status!r}, "
+                f"operating_status={self.operating_status!r}, "
+                f"provider={self.provider!r})")
+
 
 class VRRPGroup(base_models.BASE):
 
@@ -468,6 +501,28 @@ class Vip(base_models.BASE):
     octavia_owned = sa.Column(sa.Boolean(), nullable=True)
 
 
+class AdditionalVip(base_models.BASE):
+
+    __data_model__ = data_models.AdditionalVip
+
+    __tablename__ = "additional_vip"
+
+    __table_args__ = (
+        sa.PrimaryKeyConstraint('load_balancer_id', 'subnet_id',
+                                name='pk_add_vip_load_balancer_subnet'),
+    )
+
+    load_balancer_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("load_balancer.id",
+                      name="fk_add_vip_load_balancer_id"),
+        nullable=False, index=True)
+    ip_address = sa.Column(sa.String(64), nullable=True)
+    port_id = sa.Column(sa.String(36), nullable=True)
+    subnet_id = sa.Column(sa.String(36), nullable=True)
+    network_id = sa.Column(sa.String(36), nullable=True)
+
+
 class Listener(base_models.BASE, base_models.IdMixin,
                base_models.ProjectMixin, models.TimestampMixin,
                base_models.NameMixin, base_models.TagMixin):
@@ -573,6 +628,13 @@ class Listener(base_models.BASE, base_models.IdMixin,
         'ListenerCidr', cascade='all,delete-orphan',
         uselist=True, backref=orm.backref('listener', uselist=False))
 
+    def __str__(self):
+        return (f"Listener(id={self.id!r}, "
+                f"default_pool={self.default_pool!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, protocol={self.protocol!r}, "
+                f"protocol_port={self.protocol_port!r}, "
+                f"enabled={self.enabled!r})")
+
 
 class SNI(base_models.BASE):
 
@@ -629,6 +691,12 @@ class Amphora(base_models.BASE, base_models.IdMixin, models.TimestampMixin):
                                      back_populates='amphorae')
     compute_flavor = sa.Column(sa.String(255), nullable=True)
 
+    def __str__(self):
+        return (f"Amphora(id={self.id!r}, load_balancer_id="
+                f"{self.load_balancer_id!r}, status={self.status!r}, "
+                f"role={self.role!r}, lb_network_ip={self.lb_network_ip!r}, "
+                f"vrrp_ip={self.vrrp_ip!r})")
+
 
 class AmphoraHealth(base_models.BASE):
     __data_model__ = data_models.AmphoraHealth
@@ -691,6 +759,12 @@ class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         primaryjoin='and_(foreign(Tags.resource_id)==L7Rule.id)'
     )
 
+    def __str__(self):
+        return (f"L7Rule(id={self.id!r}, project_id={self.project_id!r}, "
+                f"provisioning_status={self.provisioning_status!r}, "
+                f"type={self.type!r}, key={self.key!r}, value={self.value!r}, "
+                f"invert={self.invert!r}, enabled={self.enabled!r})")
+
 
 class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
                models.TimestampMixin, base_models.NameMixin,
@@ -750,6 +824,13 @@ class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         primaryjoin='and_(foreign(Tags.resource_id)==L7Policy.id)'
     )
 
+    def __str__(self):
+        return (f"L7Policy(id={self.id!r}, name={self.name!r}, "
+                f"project_id={self.project_id!r}, "
+                f"provisioning_status={self.provisioning_status!r}, "
+                f"action={self.action!r}, position={self.position!r}, "
+                f"enabled={self.enabled!r})")
+
 
 class Quotas(base_models.BASE):
 
@@ -775,6 +856,14 @@ class Quotas(base_models.BASE):
     in_use_l7policy = sa.Column(sa.Integer(), nullable=True)
     in_use_l7rule = sa.Column(sa.Integer(), nullable=True)
 
+    def __str__(self):
+        return (f"Quotas(project_id={self.project_id!r}, "
+                f"load_balancer={self.load_balancer!r}, "
+                f"listener={self.listener!r}, pool={self.pool!r}, "
+                f"health_monitor={self.health_monitor!r}, "
+                f"member={self.member!r}, l7policy={self.l7policy!r}, "
+                f"l7rule={self.l7rule!r})")
+
 
 class FlavorProfile(base_models.BASE, base_models.IdMixin,
                     base_models.NameMixin):
@@ -855,13 +944,6 @@ class ClientAuthenticationMode(base_models.BASE):
     name = sa.Column(sa.String(10), primary_key=True, nullable=False)
 
 
-class SparesPool(base_models.BASE):
-
-    __tablename__ = "spares_pool"
-
-    updated_at = sa.Column(sa.DateTime, primary_key=True, nullable=True)
-
-
 class ListenerCidr(base_models.BASE):
 
     __data_model__ = data_models.ListenerCidr
