@@ -29,11 +29,14 @@ class Interface(data_models.BaseDataModel):
 class Delta(data_models.BaseDataModel):
 
     def __init__(self, amphora_id=None, compute_id=None,
-                 add_nics=None, delete_nics=None):
+                 add_nics=None, delete_nics=None,
+                 add_subnets=None, delete_subnets=None):
         self.compute_id = compute_id
         self.amphora_id = amphora_id
         self.add_nics = add_nics
         self.delete_nics = delete_nics
+        self.add_subnets = add_subnets
+        self.delete_subnets = delete_subnets
 
 
 class Network(data_models.BaseDataModel):
@@ -43,7 +46,8 @@ class Network(data_models.BaseDataModel):
                  provider_network_type=None,
                  provider_physical_network=None,
                  provider_segmentation_id=None,
-                 router_external=None):
+                 router_external=None,
+                 port_security_enabled=None):
         self.id = id
         self.name = name
         self.subnets = subnets
@@ -54,6 +58,7 @@ class Network(data_models.BaseDataModel):
         self.provider_segmentation_id = provider_segmentation_id
         self.router_external = router_external
         self.mtu = mtu
+        self.port_security_enabled = port_security_enabled
 
 
 class Subnet(data_models.BaseDataModel):
@@ -130,7 +135,7 @@ class AmphoraNetworkConfig(data_models.BaseDataModel):
 
     def __init__(self, amphora=None, vip_subnet=None, vip_port=None,
                  vrrp_subnet=None, vrrp_port=None, ha_subnet=None,
-                 ha_port=None):
+                 ha_port=None, additional_vip_data=None):
         self.amphora = amphora
         self.vip_subnet = vip_subnet
         self.vip_port = vip_port
@@ -138,6 +143,14 @@ class AmphoraNetworkConfig(data_models.BaseDataModel):
         self.vrrp_port = vrrp_port
         self.ha_subnet = ha_subnet
         self.ha_port = ha_port
+        self.additional_vip_data = additional_vip_data or []
+
+
+class AdditionalVipData(data_models.BaseDataModel):
+
+    def __init__(self, ip_address=None, subnet=None):
+        self.ip_address = ip_address
+        self.subnet = subnet
 
 
 class HostRoute(data_models.BaseDataModel):
