@@ -123,14 +123,14 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             raise base.PlugVIPException(message) from e
         return interface
 
-    def _add_vip_address_pair(self, port_id, vip_address):
+    def _add_vip_address_pairs(self, port_id, vip_address_list):
         try:
-            self._add_allowed_address_pair_to_port(port_id, vip_address)
+            self._add_allowed_address_pairs_to_port(port_id, vip_address_list)
         except neutron_client_exceptions.PortNotFoundClient as e:
             raise base.PortNotFound(str(e))
         except Exception as e:
-            message = _('Error adding allowed address pair {ip} '
-                        'to port {port_id}.').format(ip=vip_address,
+            message = _('Error adding allowed address pair(s) {ips} '
+                        'to port {port_id}.').format(ips=vip_address_list,
                                                      port_id=port_id)
             LOG.exception(message)
             raise base.PlugVIPException(message) from e
@@ -152,6 +152,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             security_group_id=sec_grp_id)
 
         updated_ports = []
+        listener_peer_ports = []
         for listener in load_balancer.listeners:
             if (listener.provisioning_status in [constants.PENDING_DELETE,
                                                  constants.DELETED]):
@@ -171,11 +172,17 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 port = (listener.protocol_port, protocol, None)
                 updated_ports.append(port)
 
-            # As the peer port will hold the tcp connection for keepalived and
-            # haproxy session synchronization, so here the security group rule
-            # should be just related with tcp protocol only.
-            updated_ports.append(
-                (listener.peer_port, constants.PROTOCOL_TCP.lower(), None))
+            listener_peer_ports.append(listener.peer_port)
+
+        # As the peer port will hold the tcp connection for keepalived and
+        # haproxy session synchronization, so here the security group rule
+        # should be just related with tcp protocol only. To avoid adding
+        # duplicate rules, peer_port info should be added if updated_ports
+        # does not have the peer_port entry with allowed_cidr 0.0.0.0/0
+        tcp_lower = constants.PROTOCOL_TCP.lower()
+        for peer_port in listener_peer_ports:
+            if (peer_port, tcp_lower, "0.0.0.0/0") not in updated_ports:
+                updated_ports.append((peer_port, tcp_lower, None))
 
         # Just going to use port_range_max for now because we can assume that
         # port_range_max and min will be the same since this driver is
@@ -197,7 +204,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         del_ports = set(old_ports) - set(updated_ports)
         for rule in rules.get('security_group_rules', []):
             if (rule.get('protocol', '') and
-                    rule.get('protocol', '').lower() in ['tcp', 'udp'] and
+                    rule.get('protocol', '').upper() in
+                    [constants.PROTOCOL_TCP, constants.PROTOCOL_UDP,
+                     lib_consts.PROTOCOL_SCTP] and
                     (rule.get('port_range_max'), rule.get('protocol'),
                      rule.get('remote_ip_prefix')) in del_ports):
                 rule_id = rule.get(constants.ID)
@@ -207,13 +216,19 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     LOG.info("Security group rule %s not found, will assume "
                              "it is already deleted.", rule_id)
 
-        ethertype = self._get_ethertype_for_ip(load_balancer.vip.ip_address)
+        ethertypes = set()
+        primary_ethertype = self._get_ethertype_for_ip(
+            load_balancer.vip.ip_address)
+        ethertypes.add(primary_ethertype)
+        for add_vip in load_balancer.additional_vips:
+            ethertypes.add(self._get_ethertype_for_ip(add_vip.ip_address))
         for port_protocol in add_ports:
-            self._create_security_group_rule(sec_grp_id, port_protocol[1],
-                                             port_min=port_protocol[0],
-                                             port_max=port_protocol[0],
-                                             ethertype=ethertype,
-                                             cidr=port_protocol[2])
+            for ethertype in ethertypes:
+                self._create_security_group_rule(sec_grp_id, port_protocol[1],
+                                                 port_min=port_protocol[0],
+                                                 port_max=port_protocol[0],
+                                                 ethertype=ethertype,
+                                                 cidr=port_protocol[2])
 
         # Currently we are using the VIP network for VRRP
         # so we need to open up the protocols for it
@@ -223,7 +238,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     sec_grp_id,
                     constants.VRRP_PROTOCOL_NUM,
                     direction='ingress',
-                    ethertype=ethertype)
+                    ethertype=primary_ethertype)
             except neutron_client_exceptions.Conflict:
                 # It's ok if this rule already exists
                 pass
@@ -233,7 +248,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             try:
                 self._create_security_group_rule(
                     sec_grp_id, constants.AUTH_HEADER_PROTOCOL_NUMBER,
-                    direction='ingress', ethertype=ethertype)
+                    direction='ingress', ethertype=primary_ethertype)
             except neutron_client_exceptions.Conflict:
                 # It's ok if this rule already exists
                 pass
@@ -392,7 +407,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         if not interface:
             interface = self._plug_amphora_vip(amphora, subnet)
 
-        self._add_vip_address_pair(interface.port_id, vip.ip_address)
+        aap_address_list = [vip.ip_address]
+        for add_vip in load_balancer.additional_vips:
+            aap_address_list.append(add_vip.ip_address)
+        self._add_vip_address_pairs(interface.port_id, aap_address_list)
+
         if self.sec_grp_enabled:
             self._add_vip_security_group_to_port(load_balancer.id,
                                                  interface.port_id)
@@ -496,6 +515,22 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         if load_balancer.vip.ip_address:
             fixed_ip[constants.IP_ADDRESS] = load_balancer.vip.ip_address
 
+        fixed_ips = []
+        if fixed_ip:
+            fixed_ips.append(fixed_ip)
+
+        for add_vip in load_balancer.additional_vips:
+            add_ip = {}
+            if add_vip.subnet_id:
+                add_ip['subnet_id'] = add_vip.subnet_id
+            if add_vip.ip_address:
+                add_ip['ip_address'] = add_vip.ip_address
+            if add_ip:
+                fixed_ips.append(add_ip)
+            else:
+                LOG.warning('Additional VIP contains neither subnet_id nor '
+                            'ip_address, ignoring.')
+
         # Make sure we are backward compatible with older neutron
         if self._check_extension_enabled(PROJECT_ID_ALIAS):
             project_id_key = 'project_id'
@@ -511,8 +546,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             constants.DEVICE_OWNER: OCTAVIA_OWNER,
             project_id_key: load_balancer.project_id}}
 
-        if fixed_ip:
-            port[constants.PORT][constants.FIXED_IPS] = [fixed_ip]
+        if fixed_ips:
+            port[constants.PORT][constants.FIXED_IPS] = fixed_ips
         try:
             new_port = self.neutron_client.create_port(port)
         except Exception as e:
@@ -580,11 +615,10 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 load_balancer.amphorae):
             self.unplug_aap_port(vip, amphora, subnet)
 
-    def plug_network(self, compute_id, network_id, ip_address=None):
+    def plug_network(self, compute_id, network_id):
         try:
             interface = self.compute.attach_network_or_port(
-                compute_id=compute_id, network_id=network_id,
-                ip_address=ip_address)
+                compute_id=compute_id, network_id=network_id)
         except exceptions.NotFound as e:
             if 'Instance' in str(e):
                 raise base.AmphoraNotFound(str(e))
@@ -601,18 +635,34 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         return self._nova_interface_to_octavia_interface(compute_id, interface)
 
-    def unplug_network(self, compute_id, network_id, ip_address=None):
+    def unplug_network(self, compute_id, network_id):
         interfaces = self.get_plugged_networks(compute_id)
         if not interfaces:
             msg = ('Amphora with compute id {compute_id} does not have any '
                    'plugged networks').format(compute_id=compute_id)
             raise base.NetworkNotFound(msg)
 
-        unpluggers = self._get_interfaces_to_unplug(interfaces, network_id,
-                                                    ip_address=ip_address)
+        unpluggers = self._get_interfaces_to_unplug(interfaces, network_id)
+        removed_port_ids = set()
         for index, unplugger in enumerate(unpluggers):
             self.compute.detach_port(
                 compute_id=compute_id, port_id=unplugger.port_id)
+            removed_port_ids.add(unplugger.port_id)
+
+        port_detach_timeout = CONF.networking.port_detach_timeout
+
+        start = time.time()
+        while time.time() - start < port_detach_timeout:
+            interfaces = self.get_plugged_networks(compute_id)
+            plugged_port_ids = {i.port_id for i in interfaces}
+            if not plugged_port_ids & removed_port_ids:
+                break
+            time.sleep(CONF.networking.retry_interval)
+        else:
+            LOG.warning("Ports (%s) still attached to compute %s after "
+                        "%s seconds.",
+                        ", ".join(removed_port_ids),
+                        compute_id, port_detach_timeout)
 
     def update_vip(self, load_balancer, for_delete=False):
         sec_grp = self._get_lb_security_group(load_balancer.id)
@@ -685,7 +735,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         return plugged_interface
 
-    def _get_amp_net_configs(self, amp, amp_configs, vip_subnet, vip_port):
+    def _get_amp_net_configs(self, amp, amp_configs, vip_subnet, vip_port,
+                             additional_vips):
         if amp.status != constants.DELETED:
             LOG.debug("Retrieving network details for amphora %s", amp.id)
             vrrp_port = self.get_port(amp.vrrp_port_id)
@@ -696,6 +747,15 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             ha_subnet = self.get_subnet(
                 ha_port.get_subnet_id(amp.ha_ip))
 
+            additional_vip_data = []
+            for add_vip in additional_vips:
+                add_vip_subnet = self.get_subnet(add_vip.subnet_id)
+                add_vip_data = n_data_models.AdditionalVipData(
+                    ip_address=add_vip.ip_address,
+                    subnet=add_vip_subnet
+                )
+                additional_vip_data.append(add_vip_data)
+
             amp_configs[amp.id] = n_data_models.AmphoraNetworkConfig(
                 amphora=amp,
                 vip_subnet=vip_subnet,
@@ -703,7 +763,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 vrrp_subnet=vrrp_subnet,
                 vrrp_port=vrrp_port,
                 ha_subnet=ha_subnet,
-                ha_port=ha_port
+                ha_port=ha_port,
+                additional_vip_data=additional_vip_data
             )
 
     def get_network_configs(self, loadbalancer, amphora=None):
@@ -712,12 +773,14 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         amp_configs = {}
         if amphora:
             self._get_amp_net_configs(amphora, amp_configs,
-                                      vip_subnet, vip_port)
+                                      vip_subnet, vip_port,
+                                      loadbalancer.additional_vips)
         else:
             for amp in loadbalancer.amphorae:
                 try:
                     self._get_amp_net_configs(amp, amp_configs,
-                                              vip_subnet, vip_port)
+                                              vip_subnet, vip_port,
+                                              loadbalancer.additional_vips)
                 except Exception as e:
                     LOG.warning('Getting network configurations for amphora '
                                 '%(amp)s failed due to %(err)s.',
