@@ -11,15 +11,17 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 
 from octavia.common import constants
 import octavia.common.context
+from octavia.common import data_models
 from octavia.common import exceptions
+from octavia.db import repositories
 from octavia.tests.functional.api.v2 import base
 
 
@@ -30,7 +32,7 @@ class TestL7Rule(base.BaseAPITest):
     root_tag_links = 'rules_links'
 
     def setUp(self):
-        super(TestL7Rule, self).setUp()
+        super().setUp()
         self.lb = self.create_load_balancer(uuidutils.generate_uuid())
         self.lb_id = self.lb.get('loadbalancer').get('id')
         self.project_id = self.lb.get('loadbalancer').get('project_id')
@@ -46,12 +48,13 @@ class TestL7Rule(base.BaseAPITest):
         self.l7rules_path = self.L7RULES_PATH.format(
             l7policy_id=self.l7policy_id)
         self.l7rule_path = self.l7rules_path + '/{l7rule_id}'
+        self.l7policy_repo = repositories.L7PolicyRepository()
 
     def test_get(self):
         l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
-            '/api').get(self.root_tag)
+            '/api', tags=['test_tag']).get(self.root_tag)
         response = self.get(self.l7rule_path.format(
             l7rule_id=l7rule.get('id'))).json.get(self.root_tag)
         self.assertEqual(l7rule, response)
@@ -128,20 +131,23 @@ class TestL7Rule(base.BaseAPITest):
         api_l7r_a = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
-            '/api').get(self.root_tag)
+            '/api', tags=['test_tag1']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         api_l7r_b = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_COOKIE,
             constants.L7RULE_COMPARE_TYPE_CONTAINS, 'some-value',
-            key='some-cookie').get(self.root_tag)
+            key='some-cookie', tags=['test_tag2']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         rules = self.get(self.l7rules_path).json.get(self.root_tag_list)
         self.assertIsInstance(rules, list)
         self.assertEqual(2, len(rules))
-        rule_id_types = [(r.get('id'), r.get('type')) for r in rules]
-        self.assertIn((api_l7r_a.get('id'), api_l7r_a.get('type')),
+        rule_id_types = [(r.get('id'), r.get('type'),
+                          r['tags']) for r in rules]
+        self.assertIn((api_l7r_a.get('id'), api_l7r_a.get('type'),
+                       api_l7r_a['tags']),
                       rule_id_types)
-        self.assertIn((api_l7r_b.get('id'), api_l7r_b.get('type')),
+        self.assertIn((api_l7r_b.get('id'), api_l7r_b.get('type'),
+                       api_l7r_b['tags']),
                       rule_id_types)
 
     def test_get_all_authorized(self):
@@ -188,6 +194,42 @@ class TestL7Rule(base.BaseAPITest):
         self.assertIn((api_l7r_b.get('id'), api_l7r_b.get('type')),
                       rule_id_types)
 
+    def test_get_all_unscoped_token(self):
+        self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_COOKIE,
+            constants.L7RULE_COMPARE_TYPE_CONTAINS, 'some-value',
+            key='some-cookie').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
+        self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
+        with mock.patch.object(octavia.common.context.Context, 'project_id',
+                               None):
+            override_credentials = {
+                'service_user_id': None,
+                'user_domain_id': None,
+                'is_admin_project': True,
+                'service_project_domain_id': None,
+                'service_project_id': None,
+                'roles': ['load-balancer_member'],
+                'user_id': None,
+                'is_admin': False,
+                'service_user_domain_id': None,
+                'project_domain_id': None,
+                'service_roles': [],
+                'project_id': None}
+            with mock.patch(
+                    "oslo_context.context.RequestContext.to_policy_values",
+                    return_value=override_credentials):
+                result = self.get(self.l7rules_path, status=403).json
+        self.conf.config(group='api_settings', auth_strategy=auth_strategy)
+        self.assertEqual(self.NOT_AUTHORIZED_BODY, result)
+
     def test_get_all_not_authorized(self):
         self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
@@ -286,7 +328,8 @@ class TestL7Rule(base.BaseAPITest):
         links = middle[self.root_tag_links]
         self.assertEqual(1, len(objs))
         self.assertEqual(2, len(links))
-        self.assertItemsEqual(['previous', 'next'], [l['rel'] for l in links])
+        self.assertCountEqual(['previous', 'next'],
+                              [link['rel'] for link in links])
 
     def test_get_all_fields_filter(self):
         self.create_l7rule(
@@ -350,6 +393,95 @@ class TestL7Rule(base.BaseAPITest):
         self.assertEqual(ru1['id'],
                          l7rus['rules'][0]['id'])
 
+    def test_get_all_tags_filter(self):
+        rule1 = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api', tags=['test_tag1', 'test_tag2']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        rule2 = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_COOKIE,
+            constants.L7RULE_COMPARE_TYPE_CONTAINS, 'some-value',
+            key='some-cookie',
+            tags=['test_tag2', 'test_tag3']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        rule3 = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_HOST_NAME,
+            constants.L7RULE_COMPARE_TYPE_EQUAL_TO,
+            'www.example.com',
+            tags=['test_tag4', 'test_tag5']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(2, len(rules))
+        self.assertEqual(
+            [rule1.get('id'), rule2.get('id')],
+            [rule.get('id') for rule in rules]
+        )
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'tags': ['test_tag2', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(1, len(rules))
+        self.assertEqual(
+            [rule2.get('id')],
+            [rule.get('id') for rule in rules]
+        )
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'tags-any': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(2, len(rules))
+        self.assertEqual(
+            [rule1.get('id'), rule2.get('id')],
+            [rule.get('id') for rule in rules]
+        )
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(1, len(rules))
+        self.assertEqual(
+            [rule3.get('id')],
+            [rule.get('id') for rule in rules]
+        )
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'not-tags-any': ['test_tag2', 'test_tag4']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(0, len(rules))
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'tags': 'test_tag2',
+                    'tags-any': ['test_tag1', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(2, len(rules))
+        self.assertEqual(
+            [rule1.get('id'), rule2.get('id')],
+            [rule.get('id') for rule in rules]
+        )
+
+        rules = self.get(
+            self.l7rules_path,
+            params={'tags': 'test_tag2', 'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(rules, list)
+        self.assertEqual(0, len(rules))
+
     def test_empty_get_all(self):
         response = self.get(self.l7rules_path).json.get(self.root_tag_list)
         self.assertIsInstance(response, list)
@@ -449,6 +581,21 @@ class TestL7Rule(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, api_l7rule)
 
+    def test_create_l7policy_in_error(self):
+        l7policy = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REJECT)
+        l7policy_id = l7policy.get('l7policy').get('id')
+        self.set_lb_status(self.lb_id)
+        self.set_object_status(self.l7policy_repo, l7policy_id,
+                               provisioning_status=constants.ERROR)
+        api_l7rule = self.create_l7rule(
+            l7policy_id, constants.L7RULE_TYPE_HOST_NAME,
+            constants.L7RULE_COMPARE_TYPE_EQUAL_TO,
+            'www.example.com', status=409)
+        ref_msg = ('L7Policy %s is immutable and cannot be updated.' %
+                   l7policy_id)
+        self.assertEqual(ref_msg, api_l7rule.get('faultstring'))
+
     def test_create_path_rule(self):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
@@ -590,17 +737,156 @@ class TestL7Rule(base.BaseAPITest):
         self.assertIn('Provider \'bad_driver\' reports error: broken',
                       response.json.get('faultstring'))
 
+    def test_create_with_ssl_rule_types(self):
+        test_mapping = {
+            constants.L7RULE_TYPE_SSL_CONN_HAS_CERT: {
+                'value': 'tRuE',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO},
+            constants.L7RULE_TYPE_SSL_VERIFY_RESULT: {
+                'value': '0',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO},
+            constants.L7RULE_TYPE_SSL_DN_FIELD: {
+                'key': 'st-1', 'value': 'ST-FIELD1-PREFIX',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_STARTS_WITH}
+        }
+        for l7rule_type, test_body in test_mapping.items():
+            self.set_lb_status(self.lb_id)
+            test_body.update({'type': l7rule_type})
+            api_l7rule = self.create_l7rule(
+                self.l7policy_id, l7rule_type,
+                test_body['compare_type'], test_body['value'],
+                key=test_body.get('key')).get(self.root_tag)
+            self.assertEqual(l7rule_type, api_l7rule.get('type'))
+            self.assertEqual(test_body['compare_type'],
+                             api_l7rule.get('compare_type'))
+            self.assertEqual(test_body['value'], api_l7rule.get('value'))
+            if test_body.get('key'):
+                self.assertEqual(test_body['key'], api_l7rule.get('key'))
+            self.assertFalse(api_l7rule.get('invert'))
+            self.assert_correct_status(
+                lb_id=self.lb_id, listener_id=self.listener_id,
+                l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
+                lb_prov_status=constants.PENDING_UPDATE,
+                listener_prov_status=constants.PENDING_UPDATE,
+                l7policy_prov_status=constants.PENDING_UPDATE,
+                l7rule_prov_status=constants.PENDING_CREATE,
+                l7rule_op_status=constants.OFFLINE)
+
+    def _test_bad_cases_with_ssl_rule_types(self, is_create=True,
+                                            rule_id=None):
+        if is_create:
+            req_func = self.post
+            first_req_arg = self.l7rules_path
+        else:
+            req_func = self.put
+            first_req_arg = self.l7rule_path.format(l7rule_id=rule_id)
+
+        # test bad cases of L7RULE_TYPE_SSL_CONN_HAS_CERT
+        l7rule = {'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO,
+                  'invert': False,
+                  'type': constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+                  'value': 'true',
+                  'admin_state_up': True,
+                  'key': 'no-need-key'}
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn('L7rule type {0} does not use the "key" field.'.format(
+            constants.L7RULE_TYPE_SSL_CONN_HAS_CERT),
+            response.get('faultstring'))
+
+        l7rule.pop('key')
+        l7rule['value'] = 'not-true-string'
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule value {0} is not a boolean True string.'.format(
+                l7rule['value']), response.get('faultstring'))
+
+        l7rule['value'] = 'tRUe'
+        l7rule['compare_type'] = constants.L7RULE_COMPARE_TYPE_STARTS_WITH
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule type {0} only supports the {1} compare type.'.format(
+                constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+                constants.L7RULE_COMPARE_TYPE_EQUAL_TO),
+            response.get('faultstring'))
+
+        # test bad cases of L7RULE_TYPE_SSL_VERIFY_RES
+        l7rule = {'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO,
+                  'invert': False,
+                  'type': constants.L7RULE_TYPE_SSL_VERIFY_RESULT,
+                  'value': 'true',
+                  'admin_state_up': True,
+                  'key': 'no-need-key'}
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule type {0} does not use the "key" field.'.format(
+                l7rule['type']), response.get('faultstring'))
+
+        l7rule.pop('key')
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule type {0} needs a int value, which is >= 0'.format(
+                l7rule['type']), response.get('faultstring'))
+
+        l7rule['value'] = '0'
+        l7rule['compare_type'] = constants.L7RULE_COMPARE_TYPE_STARTS_WITH
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule type {0} only supports the {1} compare type.'.format(
+                l7rule['type'], constants.L7RULE_COMPARE_TYPE_EQUAL_TO),
+            response.get('faultstring'))
+
+        # test bad cases of L7RULE_TYPE_SSL_DN_FIELD
+        l7rule = {'compare_type': constants.L7RULE_COMPARE_TYPE_REGEX,
+                  'invert': False,
+                  'type': constants.L7RULE_TYPE_SSL_DN_FIELD,
+                  'value': 'bad regex\\',
+                  'admin_state_up': True}
+        # This case just test that fail to parse the regex from the value
+        req_func(first_req_arg, self._build_body(l7rule), status=400).json
+
+        l7rule['value'] = '^.test*$'
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn(
+            'L7rule type {0} needs to specify a key and a value.'.format(
+                l7rule['type']), response.get('faultstring'))
+
+        l7rule['key'] = 'NOT_SUPPORTED_DN_FIELD'
+        response = req_func(first_req_arg, self._build_body(l7rule),
+                            status=400).json
+        self.assertIn('Invalid L7rule distinguished name field.',
+                      response.get('faultstring'))
+
+    def test_create_bad_cases_with_ssl_rule_types(self):
+        self._test_bad_cases_with_ssl_rule_types()
+
+    def test_create_over_quota(self):
+        self.start_quota_mock(data_models.L7Rule)
+        l7rule = {'compare_type': 'REGEX',
+                  'invert': False,
+                  'type': 'PATH',
+                  'value': '/images*',
+                  'admin_state_up': True}
+        self.post(self.l7rules_path, self._build_body(l7rule), status=403)
+
     def test_update(self):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
-            '/api').get(self.root_tag)
+            '/api', tags=['old_tag']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
-        new_l7rule = {'value': '/images'}
+        new_l7rule = {'value': '/images', 'tags': ['new_tag']}
         response = self.put(self.l7rule_path.format(
             l7rule_id=api_l7rule.get('id')),
             self._build_body(new_l7rule)).json.get(self.root_tag)
         self.assertEqual('/images', response.get('value'))
+        self.assertEqual(['new_tag'], response['tags'])
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
             l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
@@ -718,6 +1004,65 @@ class TestL7Rule(base.BaseAPITest):
             l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
             l7rule_prov_status=constants.ACTIVE)
 
+    def test_update_with_ssl_rule_types(self):
+        test_mapping = {
+            constants.L7RULE_TYPE_SSL_CONN_HAS_CERT: {
+                'value': 'tRuE',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO},
+            constants.L7RULE_TYPE_SSL_VERIFY_RESULT: {
+                'value': '0',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO},
+            constants.L7RULE_TYPE_SSL_DN_FIELD: {
+                'key': 'st-1', 'value': 'ST-FIELD1-PREFIX',
+                'compare_type': constants.L7RULE_COMPARE_TYPE_STARTS_WITH}
+        }
+
+        for l7rule_type, test_body in test_mapping.items():
+            self.set_lb_status(self.lb_id)
+            api_l7rule = self.create_l7rule(
+                self.l7policy_id, constants.L7RULE_TYPE_PATH,
+                constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+                '/api').get(self.root_tag)
+            self.set_lb_status(self.lb_id)
+            test_body.update({'type': l7rule_type})
+            response = self.put(self.l7rule_path.format(
+                l7rule_id=api_l7rule.get('id')),
+                self._build_body(test_body)).json.get(self.root_tag)
+            self.assertEqual(l7rule_type, response.get('type'))
+            self.assertEqual(test_body['compare_type'],
+                             response.get('compare_type'))
+            self.assertEqual(test_body['value'], response.get('value'))
+            if test_body.get('key'):
+                self.assertEqual(test_body['key'], response.get('key'))
+            self.assertFalse(response.get('invert'))
+            self.assert_correct_status(
+                lb_id=self.lb_id, listener_id=self.listener_id,
+                l7policy_id=self.l7policy_id, l7rule_id=response.get('id'),
+                lb_prov_status=constants.PENDING_UPDATE,
+                listener_prov_status=constants.PENDING_UPDATE,
+                l7policy_prov_status=constants.PENDING_UPDATE,
+                l7rule_prov_status=constants.PENDING_UPDATE)
+
+    def test_update_bad_cases_with_ssl_rule_types(self):
+        api_l7rule = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api').get(self.root_tag)
+        self._test_bad_cases_with_ssl_rule_types(
+            is_create=False, rule_id=api_l7rule.get('id'))
+
+    def test_update_invert_none(self):
+        api_l7rule = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api', tags=['old_tag'], invert=True).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7rule = {'invert': None}
+        response = self.put(self.l7rule_path.format(
+            l7rule_id=api_l7rule.get('id')),
+            self._build_body(new_l7rule)).json.get(self.root_tag)
+        self.assertFalse(response.get('invert'))
+
     def test_delete(self):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
@@ -959,3 +1304,19 @@ class TestL7Rule(base.BaseAPITest):
         self.set_lb_status(self.lb_id, status=constants.DELETED)
         self.delete(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
                     status=404)
+
+    @mock.patch("octavia.api.drivers.noop_driver.driver.NoopManager."
+                "l7rule_create")
+    def test_create_with_exception_in_provider_driver(self,
+                                                      l7rule_create_mock):
+        l7rule_create_mock.side_effect = Exception("Provider error")
+
+        self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api', status=500)
+
+        lb = self.get(self.LB_PATH.format(lb_id=self.lb_id)).json.get(
+            "loadbalancer")
+        self.assertEqual(lb[constants.PROVISIONING_STATUS],
+                         constants.ACTIVE)
