@@ -17,6 +17,7 @@ from unittest import mock
 
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
+from oslo_utils.secretutils import md5
 from oslo_utils import uuidutils
 import requests
 import requests_mock
@@ -41,7 +42,8 @@ FAKE_IPV6 = '2001:db8::cafe'
 FAKE_IPV6_LLA = 'fe80::00ff:fe00:cafe'
 FAKE_PEM_FILENAME = "file_name"
 FAKE_UUID_1 = uuidutils.generate_uuid()
-FAKE_VRRP_IP = '10.1.0.1'
+FAKE_VRRP_IP = '192.0.2.5'
+FAKE_VIP_SUBNET = '192.0.2.0/24'
 FAKE_MAC_ADDRESS = '123'
 FAKE_MTU = 1450
 FAKE_MEMBER_IP_PORT_NAME_1 = "10.0.0.10:1003"
@@ -71,7 +73,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             'haproxy_version': u'1.6.3-1ubuntu0.1',
             'api_version': API_VERSION}
         self.driver.jinja_combo = mock.MagicMock()
-        self.driver.udp_jinja = mock.MagicMock()
+        self.driver.lvs_jinja = mock.MagicMock()
 
         # Build sample Listener and VIP configs
         self.sl = sample_configs_combined.sample_listener_tuple(
@@ -93,6 +95,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.fixed_ip = mock.MagicMock()
         self.fixed_ip.ip_address = '198.51.100.5'
         self.fixed_ip.subnet.cidr = '198.51.100.0/24'
+        self.fixed_ip.subnet.gateway_ip = FAKE_GATEWAY
         self.network = network_models.Network(mtu=FAKE_MTU)
         self.port = network_models.Port(mac_address=FAKE_MAC_ADDRESS,
                                         fixed_ips=[self.fixed_ip],
@@ -109,12 +112,18 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                             'mac_address': FAKE_MAC_ADDRESS,
                             'vrrp_ip': self.amp.vrrp_ip,
                             'mtu': FAKE_MTU,
-                            'host_routes': host_routes_data}
+                            'host_routes': host_routes_data,
+                            'additional_vips': []}
 
         self.timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
                              constants.REQ_READ_TIMEOUT: 2,
                              constants.CONN_MAX_RETRIES: 3,
                              constants.CONN_RETRY_INTERVAL: 4}
+        self.amp_net_config = network_models.AmphoraNetworkConfig(
+            vip_subnet=network_models.Subnet(
+                id=self.lb.vip.subnet_id,
+                cidr=FAKE_VIP_SUBNET,
+                host_routes=[]))
 
     @mock.patch('octavia.amphorae.drivers.haproxy.rest_api_driver.'
                 'HaproxyAmphoraLoadBalancerDriver._process_secret')
@@ -253,7 +262,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_secret.assert_has_calls(secret_calls)
 
     def test_udp_update(self):
-        self.driver.udp_jinja.build_config.side_effect = ['fake_udp_config']
+        self.driver.lvs_jinja.build_config.side_effect = ['fake_udp_config']
 
         # Execute driver method
         self.driver.update(self.lb_udp)
@@ -343,7 +352,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_oslo.return_value = fake_context
         self.driver.cert_manager.get_secret.reset_mock()
         self.driver.cert_manager.get_secret.return_value = fake_secret
-        ref_md5 = hashlib.md5(fake_secret).hexdigest()  # nosec
+        ref_md5 = md5(fake_secret, usedforsecurity=False).hexdigest()  # nosec
         ref_id = hashlib.sha1(fake_secret).hexdigest()  # nosec
         ref_name = '{id}.pem'.format(id=ref_id)
 
@@ -357,7 +366,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             fake_context, sample_listener.client_ca_tls_certificate_id)
         mock_upload_cert.assert_called_once_with(
             self.amp, sample_listener.id, pem=fake_secret,
-            md5=ref_md5, name=ref_name)
+            md5sum=ref_md5, name=ref_name)
         self.assertEqual(ref_name, result)
 
     @mock.patch('octavia.amphorae.drivers.haproxy.rest_api_driver.'
@@ -407,7 +416,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_load_certs.return_value = pool_data
         fake_pem = b'fake pem'
         mock_build_pem.return_value = fake_pem
-        ref_md5 = hashlib.md5(fake_pem).hexdigest()  # nosec
+        ref_md5 = md5(fake_pem, usedforsecurity=False).hexdigest()  # nosec
         ref_name = '{id}.pem'.format(id=pool_cert.id)
         ref_path = '{cert_dir}/{lb_id}/{name}'.format(
             cert_dir=fake_cert_dir, lb_id=sample_listener.load_balancer.id,
@@ -439,7 +448,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_build_pem.assert_called_once_with(pool_cert)
         mock_upload_cert.assert_called_once_with(
             self.amp, sample_listener.load_balancer.id, pem=fake_pem,
-            md5=ref_md5, name=ref_name)
+            md5sum=ref_md5, name=ref_name)
         mock_secret.assert_has_calls(secret_calls)
         self.assertEqual(ref_result, result)
 
@@ -489,6 +498,16 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             API_VERSION].reload_listener.assert_called_once_with(
             amp1, loadbalancer.id, None)
 
+        self.driver.clients[
+            API_VERSION].reload_listener.reset_mock()
+        timeout_dict = {
+            'elem1': 1000
+        }
+        self.driver.reload(loadbalancer, timeout_dict=timeout_dict)
+        self.driver.clients[
+            API_VERSION].reload_listener.assert_called_once_with(
+            amp1, loadbalancer.id, timeout_dict)
+
     def test_start_with_amphora(self):
         # Execute driver method
         amp = mock.MagicMock()
@@ -683,17 +702,57 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         amphorae_network_config.get().vip_subnet.cidr = FAKE_CIDR
         amphorae_network_config.get().vip_subnet.gateway_ip = FAKE_GATEWAY
         amphorae_network_config.get().vip_subnet.host_routes = self.host_routes
+        amphorae_network_config.get().vip_subnet.to_dict.return_value = {
+            'cidr': FAKE_CIDR,
+            'gateway_ip': FAKE_GATEWAY,
+            'host_routes': [
+                hr.to_dict(recurse=True)
+                for hr in self.host_routes]
+        }
         amphorae_network_config.get().vrrp_port = self.port
         self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config)
         self.driver.clients[API_VERSION].plug_vip.assert_called_once_with(
             self.amp, self.lb.vip.ip_address, self.subnet_info)
 
+    def test_post_vip_plug_additional_vips(self):
+        amphorae_network_config = mock.MagicMock()
+        amphorae_network_config.get().vip_subnet.cidr = FAKE_CIDR
+        amphorae_network_config.get().vip_subnet.gateway_ip = FAKE_GATEWAY
+        amphorae_network_config.get().vip_subnet.host_routes = self.host_routes
+        amphorae_network_config.get().vip_subnet.to_dict.return_value = {
+            'cidr': FAKE_CIDR,
+            'gateway_ip': FAKE_GATEWAY,
+            'host_routes': [
+                hr.to_dict(recurse=True)
+                for hr in self.host_routes]
+        }
+        amphorae_network_config.get().vrrp_port = self.port
+        additional_vip1 = mock.MagicMock()
+        additional_vip1.ip_address = mock.Mock()
+        additional_vip1.subnet.cidr = mock.Mock()
+        additional_vip1.subnet.gateway_ip = mock.Mock()
+        additional_vip1.subnet.host_routes = self.host_routes
+        additional_vip_data = [additional_vip1]
+        self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config,
+                                  additional_vip_data=additional_vip_data)
+        netinfo = self.subnet_info.copy()
+        netinfo['additional_vips'] = [
+            {
+                'ip_address': additional_vip1.ip_address,
+                'subnet_cidr': additional_vip1.subnet.cidr,
+                'gateway': additional_vip1.subnet.gateway_ip,
+                'host_routes': netinfo['host_routes']
+            }
+        ]
+        self.driver.clients[API_VERSION].plug_vip.assert_called_once_with(
+            self.amp, self.lb.vip.ip_address, netinfo)
+
     def test_post_network_plug(self):
         # Test dhcp path
         port = network_models.Port(mac_address=FAKE_MAC_ADDRESS,
                                    fixed_ips=[],
                                    network=self.network)
-        self.driver.post_network_plug(self.amp, port)
+        self.driver.post_network_plug(self.amp, port, self.amp_net_config)
         self.driver.clients[API_VERSION].plug_network.assert_called_once_with(
             self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
                            fixed_ips=[],
@@ -702,14 +761,41 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.clients[API_VERSION].plug_network.reset_mock()
 
         # Test fixed IP path
-        self.driver.post_network_plug(self.amp, self.port)
+        self.driver.post_network_plug(self.amp, self.port, self.amp_net_config)
         self.driver.clients[API_VERSION].plug_network.assert_called_once_with(
             self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
                            fixed_ips=[dict(ip_address='198.51.100.5',
                                            subnet_cidr='198.51.100.0/24',
-                                           host_routes=[])],
+                                           host_routes=[],
+                                           gateway=FAKE_GATEWAY)],
                            mtu=FAKE_MTU))
 
+        self.driver.clients[API_VERSION].plug_network.reset_mock()
+
+        # Test member network on vip port
+        port = network_models.Port(id=self.amp.vrrp_port_id,
+                                   mac_address=FAKE_MAC_ADDRESS,
+                                   fixed_ips=[self.fixed_ip],
+                                   network=self.network)
+        self.driver.post_network_plug(self.amp, port, self.amp_net_config)
+        self.driver.clients[API_VERSION].plug_network.assert_called_once_with(
+            self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
+                           fixed_ips=[dict(ip_address='198.51.100.5',
+                                           subnet_cidr='198.51.100.0/24',
+                                           host_routes=[],
+                                           gateway=FAKE_GATEWAY)],
+                           mtu=FAKE_MTU,
+                           vip_net_info=dict(
+                               vip=self.amp.ha_ip,
+                               subnet_cidr=FAKE_VIP_SUBNET,
+                               mac_address=FAKE_MAC_ADDRESS,
+                               gateway=None,
+                               vrrp_ip=self.amp.vrrp_ip,
+                               host_routes=[],
+                               additional_vips=[],
+                               mtu=FAKE_MTU
+                           )))
+
     def test_post_network_plug_with_host_routes(self):
         SUBNET_ID = 'SUBNET_ID'
         FIXED_IP1 = '192.0.2.2'
@@ -723,6 +809,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                        network_models.HostRoute(destination=DEST2,
                                                 nexthop=NEXTHOP)]
         subnet = network_models.Subnet(id=SUBNET_ID, cidr=SUBNET_CIDR,
+                                       gateway_ip=FAKE_GATEWAY,
                                        ip_version=4, host_routes=host_routes)
         fixed_ips = [
             network_models.FixedIP(subnet_id=subnet.id, ip_address=FIXED_IP1,
@@ -733,12 +820,14 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         port = network_models.Port(mac_address=FAKE_MAC_ADDRESS,
                                    fixed_ips=fixed_ips,
                                    network=self.network)
-        self.driver.post_network_plug(self.amp, port)
+        self.driver.post_network_plug(self.amp, port, self.amp_net_config)
         expected_fixed_ips = [
             {'ip_address': FIXED_IP1, 'subnet_cidr': SUBNET_CIDR,
+             'gateway': FAKE_GATEWAY,
              'host_routes': [{'destination': DEST1, 'nexthop': NEXTHOP},
                              {'destination': DEST2, 'nexthop': NEXTHOP}]},
             {'ip_address': FIXED_IP2, 'subnet_cidr': SUBNET_CIDR,
+             'gateway': FAKE_GATEWAY,
              'host_routes': [{'destination': DEST1, 'nexthop': NEXTHOP},
                              {'destination': DEST2, 'nexthop': NEXTHOP}]}
         ]
@@ -751,7 +840,19 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         ref_haproxy_versions = ['1', '6']
         result = self.driver._get_haproxy_versions(self.amp)
         self.driver.clients[API_VERSION].get_info.assert_called_once_with(
-            self.amp)
+            self.amp, timeout_dict=None)
+        self.assertEqual(ref_haproxy_versions, result)
+
+    def test_get_haproxy_versions_with_timeout_dict(self):
+        ref_haproxy_versions = ['1', '6']
+        timeout_dict = {
+            constants.CONN_MAX_RETRIES: 100,
+            constants.CONN_RETRY_INTERVAL: 1
+        }
+        result = self.driver._get_haproxy_versions(self.amp,
+                                                   timeout_dict=timeout_dict)
+        self.driver.clients[API_VERSION].get_info.assert_called_once_with(
+            self.amp, timeout_dict=timeout_dict)
         self.assertEqual(ref_haproxy_versions, result)
 
     def test_populate_amphora_api_version(self):
@@ -839,6 +940,19 @@ class TestAmphoraAPIClientTest(base.TestCase):
         information = self.driver.get_info(self.amp)
         self.assertEqual(info, information)
 
+    @requests_mock.mock()
+    def test_get_info_with_timeout_dict(self, m):
+        info = {"hostname": "some_hostname", "version": "some_version",
+                "api_version": "0.5", "uuid": FAKE_UUID_1}
+        m.get("{base}/info".format(base=self.base_url_ver),
+              json=info)
+        timeout_dict = {
+            constants.CONN_MAX_RETRIES: 100,
+            constants.CONN_RETRY_INTERVAL: 1
+        }
+        information = self.driver.get_info(self.amp, timeout_dict=timeout_dict)
+        self.assertEqual(info, information)
+
     @requests_mock.mock()
     def test_get_info_unauthorized(self, m):
         m.get("{base}/info".format(base=self.base_url_ver),
