@@ -15,7 +15,9 @@
 
 import copy
 import os
+from unittest import mock
 
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 
@@ -39,7 +41,7 @@ class TestHaproxyCfg(base.TestCase):
         self.assertEqual('haproxy.cfg.j2', template.name)
 
     def test_render_template_tls(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -64,6 +66,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -97,7 +100,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_no_sni(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -118,6 +121,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -148,7 +152,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_no_ciphers(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -168,6 +172,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -198,7 +203,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_no_versions(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -223,6 +228,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -256,7 +262,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_no_ciphers_or_versions(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -276,6 +282,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -307,7 +314,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_alpn(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -329,6 +336,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -360,7 +368,7 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_tls_no_alpn(self):
-        conf = oslo_fixture.Config(cfg.CONF)
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
         FAKE_CRT_LIST_FILENAME = os.path.join(
             CONF.haproxy_amphora.base_cert_dir,
@@ -380,6 +388,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -415,6 +424,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -436,11 +446,79 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
+    def test_render_template_prometheus(self):
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    bind 10.0.0.2:80\n"
+              "    mode http\n"
+              "    timeout client 50000\n"
+              "    default_backend prometheus-exporter-internal\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        be = ""
+        defaults = ("defaults\n"
+                    "    log global\n"
+                    "    retries 3\n"
+                    "    option redispatch\n"
+                    "    option splice-request\n"
+                    "    option splice-response\n"
+                    "    option http-keep-alive\n\n\n\n"
+                    "frontend prometheus-exporter-internal-endpoint\n"
+                    "    bind 127.0.0.1:9101\n"
+                    "    mode http\n"
+                    "    no log\n"
+                    "    option http-use-htx\n"
+                    "    http-request use-service prometheus-exporter if { "
+                    "path /metrics }\n"
+                    "    http-request reject\n"
+                    "    timeout http-request 5s\n"
+                    "    timeout client 5s\n"
+                    "backend prometheus-exporter-internal\n"
+                    "    mode http\n"
+                    "    no log\n"
+                    "    balance first\n"
+                    "    timeout connect 5s\n"
+                    "    timeout server 5s\n"
+                    "    server prometheus-internal 127.0.0.1:9102")
+        logging = ("    log-format 12345\\ sample_loadbalancer_id_1\\ %f\\ "
+                   "%ci\\ %cp\\ %t\\ -\\ -\\ %B\\ %U\\ "
+                   "%[ssl_c_verify]\\ %{+Q}[ssl_c_s_dn]\\ %b\\ %s\\ %Tt\\ "
+                   "%tsc\n\n")
+
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto=lib_consts.PROTOCOL_PROMETHEUS, include_pools=False)],
+            feature_compatibility={lib_consts.PROTOCOL_PROMETHEUS: True})
+
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(
+                frontend=fe, backend=be, logging=logging, defaults=defaults),
+            rendered_obj)
+
+    def test_render_template_additional_vips(self):
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    bind 10.0.0.2:80\n"
+              "    bind 10.0.1.2:80\n"
+              "    bind 2001:db8::2:80\n"
+              "    mode http\n"
+              "    default_backend sample_pool_id_1:sample_listener_id_1\n"
+              "    timeout client 50000\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                additional_vips=True)])
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(frontend=fe),
+            rendered_obj)
+
     def test_render_template_member_backup(self):
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -477,6 +555,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -512,6 +591,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -540,6 +620,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -580,6 +661,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode tcp\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -616,6 +698,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode tcp\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option ssl-hello-chk\n"
               "    fullconn {maxconn}\n"
@@ -683,6 +766,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option external-check\n"
               "    external-check command /var/lib/octavia/ping-wrapper.sh\n"
@@ -706,6 +790,37 @@ class TestHaproxyCfg(base.TestCase):
         self.assertEqual(sample_configs_combined.sample_base_expected_config(
             backend=be, global_opts=go), rendered_obj)
 
+    def test_render_template_ping_monitor_http_insecure_fork(self):
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
+              "    timeout check 31s\n"
+              "    option external-check\n"
+              "    external-check command /var/lib/octavia/ping-wrapper.sh\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        go = (f"    maxconn {constants.HAPROXY_DEFAULT_MAXCONN}\n"
+              "    external-check\n    insecure-fork-wanted\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto='HTTP', monitor_proto='PING')],
+            feature_compatibility={
+                "requires_insecure_fork": True})
+        self.assertEqual(sample_configs_combined.sample_base_expected_config(
+            backend=be, global_opts=go), rendered_obj)
+
     def test_render_template_no_monitor_https(self):
         fe = ("frontend sample_listener_id_1\n"
               "    maxconn {maxconn}\n"
@@ -743,6 +858,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.1\\r\\nHost:\\ "
               "testlab.com\n"
@@ -820,6 +936,7 @@ class TestHaproxyCfg(base.TestCase):
               "    balance roundrobin\n"
               "    stick-table type ip size 10k\n"
               "    stick on src\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -847,6 +964,7 @@ class TestHaproxyCfg(base.TestCase):
               "    stick-table type string len 64 size 10k\n"
               "    stick store-response res.cook(JSESSIONID)\n"
               "    stick match req.cook(JSESSIONID)\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -970,6 +1088,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -986,6 +1105,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /healthmon.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1007,6 +1127,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1035,6 +1156,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1065,6 +1187,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    fullconn {maxconn}\n"
               "    option allbackups\n"
@@ -1085,12 +1208,55 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_pool_cert(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
+              "{opts} alpn {alpn}\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
+              "{opts} alpn {alpn}\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
+                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B +
+                 " no-sslv3 no-tlsv10 no-tlsv11",
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                pool_cert=True, tls_enabled=True,
+                backend_tls_ciphers=constants.CIPHERS_OWASP_SUITE_B)],
+            tls_certs={
+                'sample_pool_id_1':
+                    {'client_cert': cert_file_path,
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_pool_cert_no_alpn(self):
+        feature_compatibility = {constants.POOL_ALPN: False}
+        cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
+                                      'sample_listener_id_1', 'fake path')
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1116,18 +1282,21 @@ class TestHaproxyCfg(base.TestCase):
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
-                     'ca_cert': None, 'crl': None}})
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
     def test_render_template_pool_cert_no_versions(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1137,13 +1306,14 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts}\n"
+              "{opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts}\n\n").format(
+              "{opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
-                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B)
+                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
@@ -1153,18 +1323,21 @@ class TestHaproxyCfg(base.TestCase):
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
-                     'ca_cert': None, 'crl': None}})
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
     def test_render_template_pool_cert_no_ciphers(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1174,13 +1347,14 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts}\n"
+              "{opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts}\n\n").format(
+              "{opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
-                 " no-sslv3 no-tlsv10 no-tlsv11")
+                 " no-sslv3 no-tlsv10 no-tlsv11",
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
@@ -1188,18 +1362,20 @@ class TestHaproxyCfg(base.TestCase):
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
-                     'ca_cert': None, 'crl': None}})
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
-    def test_render_template_pool_cert_no_ciphers_or_versions(self):
+    def test_render_template_pool_cert_no_ciphers_or_versions_or_alpn(self):
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1218,7 +1394,8 @@ class TestHaproxyCfg(base.TestCase):
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
-                pool_cert=True, tls_enabled=True, backend_tls_versions=None)],
+                pool_cert=True, tls_enabled=True, backend_tls_versions=None,
+                backend_alpn_protocols=None)],
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
@@ -1227,7 +1404,35 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
+    def test_render_template_pool_no_alpn(self):
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2"
+              "\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                backend_alpn_protocols=None)])
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
     def test_render_template_with_full_pool_cert(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
         pool_client_cert = '/foo/cert.pem'
         pool_ca_cert = '/foo/ca.pem'
         pool_crl = '/foo/crl.pem'
@@ -1235,6 +1440,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1244,16 +1450,17 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts}\n"
+              "{opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts}\n\n").format(
+              "{opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="%s %s %s %s %s %s" % (
                 "ssl", "crt", pool_client_cert,
                 "ca-file %s" % pool_ca_cert,
                 "crl-file %s" % pool_crl,
-                "verify required sni ssl_fc_sni no-sslv3 no-tlsv10 no-tlsv11"))
+                "verify required sni ssl_fc_sni no-sslv3 no-tlsv10 no-tlsv11"),
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
@@ -1263,7 +1470,8 @@ class TestHaproxyCfg(base.TestCase):
                 'sample_pool_id_1':
                     {'client_cert': pool_client_cert,
                      'ca_cert': pool_ca_cert,
-                     'crl': pool_crl}})
+                     'crl': pool_crl}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
@@ -1326,6 +1534,15 @@ class TestHaproxyCfg(base.TestCase):
                                                  in_listener.load_balancer)
         self.assertEqual(sample_configs_combined.RET_LISTENER_L7, ret)
 
+    def test_transform_listener_PROMETHEUS(self):
+        in_listener = sample_configs_combined.sample_listener_tuple()
+        ret = self.jinja_cfg._transform_listener(
+            in_listener, None, {lib_consts.PROTOCOL_PROMETHEUS: True},
+            in_listener.load_balancer)
+        expected_config = copy.copy(sample_configs_combined.RET_LISTENER)
+        expected_config[lib_consts.PROTOCOL_PROMETHEUS] = True
+        self.assertEqual(expected_config, ret)
+
     def test_transform_loadbalancer(self):
         in_amphora = sample_configs_combined.sample_amphora_tuple()
         in_listener = sample_configs_combined.sample_listener_tuple()
@@ -1425,7 +1642,7 @@ class TestHaproxyCfg(base.TestCase):
                     "    option redispatch\n"
                     "    option splice-request\n"
                     "    option splice-response\n"
-                    "    option http-keep-alive\n\n")
+                    "    option http-keep-alive\n\n\n")
         rendered_obj = j_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple()]
@@ -1449,6 +1666,7 @@ class TestHaproxyCfg(base.TestCase):
               "    http-reuse safe\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    fullconn {maxconn}\n"
               "    option allbackups\n"
@@ -1477,6 +1695,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    fullconn {maxconn}\n"
               "    option allbackups\n"
@@ -1550,6 +1769,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1565,6 +1785,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    option httpchk GET /healthmon.html HTTP/1.0\\r\\n\n"
               "    http-check expect rstatus 418\n"
@@ -1587,3 +1808,106 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs_combined.sample_base_expected_config(
                 frontend=fe, backend=be),
             rendered_obj)
+
+    @mock.patch("octavia.common.jinja.haproxy.combined_listeners.jinja_cfg."
+                "JinjaTemplater.render_loadbalancer_obj")
+    def test_build_config(self, mock_render_loadbalancer_obj):
+        mock_amp = mock.Mock()
+        mock_listeners = mock.Mock()
+        mock_tls_certs = mock.Mock()
+        mock_socket_path = mock.Mock()
+
+        j_cfg = jinja_cfg.JinjaTemplater()
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("0", "7", "0"),
+                           socket_path=mock_socket_path)
+
+        expected_fc = {}
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("1", "6", "0"),
+                           socket_path=mock_socket_path)
+
+        expected_fc = {
+            constants.HTTP_REUSE: True
+        }
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("1", "9", "0"),
+                           socket_path=mock_socket_path)
+
+        expected_fc = {
+            constants.HTTP_REUSE: True,
+            constants.POOL_ALPN: True
+        }
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("2", "1", "1"),
+                           socket_path=mock_socket_path)
+
+        expected_fc = {
+            constants.HTTP_REUSE: True,
+            constants.POOL_ALPN: True,
+            lib_consts.PROTOCOL_PROMETHEUS: True
+        }
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("2", "2", "1"),
+                           socket_path=mock_socket_path)
+
+        expected_fc = {
+            constants.HTTP_REUSE: True,
+            constants.POOL_ALPN: True,
+            lib_consts.PROTOCOL_PROMETHEUS: True,
+            constants.INSECURE_FORK: True
+        }
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("2", "4", "0"),
+                           socket_path=mock_socket_path)
+
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
+
+        mock_render_loadbalancer_obj.reset_mock()
+
+        j_cfg.build_config(mock_amp, mock_listeners, mock_tls_certs,
+                           haproxy_versions=("3", "1", "0"),
+                           socket_path=mock_socket_path)
+
+        mock_render_loadbalancer_obj.assert_called_once_with(
+            mock_amp, mock_listeners, tls_certs=mock_tls_certs,
+            socket_path=mock_socket_path,
+            feature_compatibility=expected_fc)
