@@ -15,6 +15,7 @@
 
 import collections
 
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 
 from octavia.common import constants
@@ -23,16 +24,20 @@ from octavia.tests.common import sample_certs
 CONF = cfg.CONF
 
 
+class AmphoraTuple(collections.namedtuple(
+        'amphora', 'id, lb_network_ip, vrrp_ip, ha_ip, vrrp_port_id, '
+                   'ha_port_id, role, status, vrrp_interface,'
+                   'vrrp_priority, api_version')):
+    def to_dict(self):
+        return self._asdict()
+
+
 def sample_amphora_tuple(id='sample_amphora_id_1', lb_network_ip='10.0.1.1',
                          vrrp_ip='10.1.1.1', ha_ip='192.168.10.1',
                          vrrp_port_id='1234', ha_port_id='1234', role=None,
                          status='ACTIVE', vrrp_interface=None,
                          vrrp_priority=None, api_version='1.0'):
-    in_amphora = collections.namedtuple(
-        'amphora', 'id, lb_network_ip, vrrp_ip, ha_ip, vrrp_port_id, '
-                   'ha_port_id, role, status, vrrp_interface,'
-                   'vrrp_priority, api_version')
-    return in_amphora(
+    amp = AmphoraTuple(
         id=id,
         lb_network_ip=lb_network_ip,
         vrrp_ip=vrrp_ip,
@@ -44,6 +49,7 @@ def sample_amphora_tuple(id='sample_amphora_id_1', lb_network_ip='10.0.1.1',
         vrrp_interface=vrrp_interface,
         vrrp_priority=vrrp_priority,
         api_version=api_version)
+    return amp
 
 
 RET_PERSISTENCE = {
@@ -309,6 +315,7 @@ RET_LISTENER = {
     'timeout_member_connect': 5000,
     'timeout_member_data': 50000,
     'timeout_tcp_inspect': 0,
+    'PROMETHEUS': False,
 }
 
 RET_LISTENER_L7 = {
@@ -331,6 +338,7 @@ RET_LISTENER_L7 = {
     'timeout_member_connect': 5000,
     'timeout_member_data': 50000,
     'timeout_tcp_inspect': 0,
+    'PROMETHEUS': False,
 }
 
 RET_LISTENER_TLS = {
@@ -378,6 +386,7 @@ RET_AMPHORA = {
     'vrrp_priority': None}
 
 RET_LB = {
+    'additional_vips': [],
     'host_amphora': RET_AMPHORA,
     'id': 'sample_loadbalancer_id_1',
     'vip_address': '10.0.0.2',
@@ -389,6 +398,7 @@ RET_LB = {
     'amphorae': [sample_amphora_tuple()]}
 
 RET_LB_L7 = {
+    'additional_vips': [],
     'host_amphora': RET_AMPHORA,
     'id': 'sample_loadbalancer_id_1',
     'vip_address': '10.0.0.2',
@@ -416,19 +426,10 @@ RET_UDP_HEALTH_MONITOR = {
                           '/lvs/check/udp_check.sh')
 }
 
-UDP_HEALTH_MONITOR_NO_SCRIPT = {
-    'id': 'sample_monitor_id_1',
-    'check_script_path': None,
-    'delay': 30,
-    'enabled': True,
-    'fall_threshold': 3,
-    'timeout': 31,
-    'type': 'UDP'
-}
-
 RET_UDP_MEMBER = {
     'id': 'member_id_1',
     'address': '192.0.2.10',
+    'ip_version': 4,
     'protocol_port': 82,
     'weight': 13,
     'enabled': True,
@@ -439,6 +440,7 @@ RET_UDP_MEMBER = {
 RET_UDP_MEMBER_MONITOR_IP_PORT = {
     'id': 'member_id_1',
     'address': '192.0.2.10',
+    'ip_version': 4,
     'protocol_port': 82,
     'weight': 13,
     'enabled': True,
@@ -449,6 +451,7 @@ RET_UDP_MEMBER_MONITOR_IP_PORT = {
 UDP_MEMBER_1 = {
     'id': 'sample_member_id_1',
     'address': '10.0.0.99',
+    'ip_version': 4,
     'enabled': True,
     'protocol_port': 82,
     'weight': 13,
@@ -459,6 +462,7 @@ UDP_MEMBER_1 = {
 UDP_MEMBER_2 = {
     'id': 'sample_member_id_2',
     'address': '10.0.0.98',
+    'ip_version': 4,
     'enabled': True,
     'protocol_port': 82,
     'weight': 13,
@@ -469,8 +473,8 @@ UDP_MEMBER_2 = {
 RET_UDP_POOL = {
     'id': 'sample_pool_id_1',
     'enabled': True,
-    'health_monitor': UDP_HEALTH_MONITOR_NO_SCRIPT,
-    'lb_algorithm': 'rr',
+    'health_monitor': RET_UDP_HEALTH_MONITOR,
+    'lb_algorithm': 'wrr',
     'members': [UDP_MEMBER_1, UDP_MEMBER_2],
     'protocol': 'udp',
     'session_persistence': UDP_SOURCE_IP_BODY
@@ -482,7 +486,7 @@ RET_UDP_LISTENER = {
         'id': 'sample_pool_id_1',
         'enabled': True,
         'health_monitor': RET_UDP_HEALTH_MONITOR,
-        'lb_algorithm': 'rr',
+        'lb_algorithm': 'wrr',
         'members': [UDP_MEMBER_1, UDP_MEMBER_2],
         'protocol': 'udp',
         'session_persistence': UDP_SOURCE_IP_BODY
@@ -493,22 +497,107 @@ RET_UDP_LISTENER = {
     'protocol_port': '80'
 }
 
+SCTP_SOURCE_IP_BODY = {
+    'type': constants.SESSION_PERSISTENCE_SOURCE_IP,
+    'persistence_timeout': 33,
+    'persistence_granularity': '255.0.0.0'
+}
+
+RET_SCTP_HEALTH_MONITOR = {
+    'id': 'sample_monitor_id_1',
+    'type': lib_consts.HEALTH_MONITOR_SCTP,
+    'delay': 30,
+    'timeout': 31,
+    'enabled': True,
+    'fall_threshold': 3,
+    'check_script_path': 'amphora-health-checker sctp'
+}
+
+RET_SCTP_MEMBER = {
+    'id': 'member_id_1',
+    'address': '192.0.2.10',
+    'ip_version': 4,
+    'protocol_port': 82,
+    'weight': 13,
+    'enabled': True,
+    'monitor_address': None,
+    'monitor_port': None
+}
+
+RET_SCTP_MEMBER_MONITOR_IP_PORT = {
+    'id': 'member_id_1',
+    'address': '192.0.2.10',
+    'ip_version': 4,
+    'protocol_port': 82,
+    'weight': 13,
+    'enabled': True,
+    'monitor_address': '192.168.1.1',
+    'monitor_port': 9000
+}
+
+SCTP_MEMBER_1 = {
+    'id': 'sample_member_id_1',
+    'address': '10.0.0.99',
+    'ip_version': 4,
+    'enabled': True,
+    'protocol_port': 82,
+    'weight': 13,
+    'monitor_address': None,
+    'monitor_port': None,
+}
+
+SCTP_MEMBER_2 = {
+    'id': 'sample_member_id_2',
+    'address': '10.0.0.98',
+    'ip_version': 4,
+    'enabled': True,
+    'protocol_port': 82,
+    'weight': 13,
+    'monitor_address': None,
+    'monitor_port': None
+}
+
+RET_SCTP_POOL = {
+    'id': 'sample_pool_id_1',
+    'enabled': True,
+    'health_monitor': RET_SCTP_HEALTH_MONITOR,
+    'lb_algorithm': 'wrr',
+    'members': [SCTP_MEMBER_1, SCTP_MEMBER_2],
+    'protocol': 'sctp',
+    'session_persistence': SCTP_SOURCE_IP_BODY
+}
+
+RET_SCTP_LISTENER = {
+    'connection_limit': 98,
+    'default_pool': {
+        'id': 'sample_pool_id_1',
+        'enabled': True,
+        'health_monitor': RET_SCTP_HEALTH_MONITOR,
+        'lb_algorithm': 'wrr',
+        'members': [SCTP_MEMBER_1, SCTP_MEMBER_2],
+        'protocol': 'sctp',
+        'session_persistence': SCTP_SOURCE_IP_BODY
+    },
+    'enabled': True,
+    'id': 'sample_listener_id_1',
+    'protocol_mode': 'sctp',
+    'protocol_port': '80'
+}
+
 
 def sample_listener_loadbalancer_tuple(
-        proto=None, topology=None, enabled=True, pools=None):
-    proto = 'HTTP' if proto is None else proto
+        topology=None, enabled=True, pools=None, additional_vips=False):
     if topology and topology in ['ACTIVE_STANDBY', 'ACTIVE_ACTIVE']:
         more_amp = True
     else:
         more_amp = False
         topology = constants.TOPOLOGY_SINGLE
     in_lb = collections.namedtuple(
-        'load_balancer', 'id, name, protocol, vip, amphorae, topology, '
-        'pools, listeners, enabled, project_id')
+        'load_balancer', 'id, name, vip, amphorae, topology, '
+        'pools, listeners, enabled, project_id, additional_vips')
     return in_lb(
         id='sample_loadbalancer_id_1',
         name='test-lb',
-        protocol=proto,
         vip=sample_vip_tuple(),
         amphorae=[sample_amphora_tuple(role=constants.ROLE_MASTER),
                   sample_amphora_tuple(
@@ -522,12 +611,14 @@ def sample_listener_loadbalancer_tuple(
         listeners=[],
         enabled=enabled,
         project_id='12345',
+        additional_vips=[sample_vip_tuple('10.0.1.2'),
+                         sample_vip_tuple('2001:db8::2')]
+        if additional_vips else []
     )
 
 
 def sample_lb_with_udp_listener_tuple(
-        proto=None, topology=None, enabled=True, pools=None):
-    proto = 'HTTP' if proto is None else proto
+        topology=None, enabled=True, pools=None):
     if topology and topology in ['ACTIVE_STANDBY', 'ACTIVE_ACTIVE']:
         more_amp = True
     else:
@@ -541,12 +632,11 @@ def sample_lb_with_udp_listener_tuple(
         monitor_proto=constants.HEALTH_MONITOR_UDP_CONNECT)]
 
     in_lb = collections.namedtuple(
-        'load_balancer', 'id, name, protocol, vip, amphorae, topology, '
-        'pools, enabled, project_id, listeners')
+        'load_balancer', 'id, name, vip, amphorae, topology, '
+        'pools, enabled, project_id, listeners, additional_vips')
     return in_lb(
         id='sample_loadbalancer_id_1',
         name='test-lb',
-        protocol=proto,
         vip=sample_vip_tuple(),
         amphorae=[sample_amphora_tuple(role=constants.ROLE_MASTER),
                   sample_amphora_tuple(
@@ -559,7 +649,8 @@ def sample_lb_with_udp_listener_tuple(
         listeners=listeners,
         pools=pools or [],
         enabled=enabled,
-        project_id='12345'
+        project_id='12345',
+        additional_vips=[]
     )
 
 
@@ -578,9 +669,9 @@ def sample_vrrp_group_tuple():
         smtp_connect_timeout='')
 
 
-def sample_vip_tuple():
-    vip = collections.namedtuple('vip', 'ip_address')
-    return vip(ip_address='10.0.0.2')
+def sample_vip_tuple(ip_address='10.0.0.2', subnet_id='vip_subnet_uuid'):
+    vip = collections.namedtuple('vip', ('ip_address', 'subnet_id'))
+    return vip(ip_address=ip_address, subnet_id=subnet_id)
 
 
 def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
@@ -611,7 +702,11 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                           alpn_protocols=constants.
                           AMPHORA_SUPPORTED_ALPN_PROTOCOLS,
                           sample_default_pool=1,
-                          pool_enabled=True):
+                          pool_enabled=True,
+                          backend_alpn_protocols=constants.
+                          AMPHORA_SUPPORTED_ALPN_PROTOCOLS,
+                          include_pools=True,
+                          additional_vips=False):
     proto = 'HTTP' if proto is None else proto
     if be_proto is None:
         be_proto = 'HTTP' if proto == 'TERMINATED_HTTPS' else proto
@@ -649,7 +744,8 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
                 tls_versions=backend_tls_versions,
-                enabled=pool_enabled),
+                enabled=pool_enabled,
+                alpn_protocols=backend_alpn_protocols),
             sample_pool_tuple(
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
@@ -661,7 +757,8 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
                 tls_versions=None,
-                enabled=pool_enabled)]
+                enabled=pool_enabled,
+                alpn_protocols=backend_alpn_protocols)]
         l7policies = [
             sample_l7policy_tuple('sample_l7policy_id_1', sample_policy=1),
             sample_l7policy_tuple('sample_l7policy_id_2', sample_policy=2),
@@ -687,7 +784,8 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
                 tls_versions=backend_tls_versions,
-                enabled=pool_enabled)]
+                enabled=pool_enabled,
+                alpn_protocols=backend_alpn_protocols)]
         l7policies = []
     listener = in_listener(
         id=id,
@@ -695,7 +793,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
         protocol_port=port,
         protocol=proto,
         load_balancer=sample_listener_loadbalancer_tuple(
-            proto=proto, topology=topology, pools=pools),
+            topology=topology, pools=pools, additional_vips=additional_vips),
         peer_port=peer_port,
         default_pool=sample_pool_tuple(
             listener_id='sample_listener_id_1',
@@ -740,7 +838,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                     intermediates=sample_certs.X509_IMDS_LIST,
                     primary_cn=sample_certs.X509_CERT_CN_3))]
         if sni else [],
-        pools=pools,
+        pools=pools if include_pools else '',
         l7policies=l7policies,
         enabled=enabled,
         insert_headers=insert_headers,
@@ -803,20 +901,27 @@ def sample_pool_tuple(listener_id=None, proto=None, monitor=True,
                       tls_ciphers=constants.CIPHERS_OWASP_SUITE_B,
                       tls_versions=constants.TLS_VERSIONS_OWASP_SUITE_B,
                       lb_algorithm=constants.LB_ALGORITHM_ROUND_ROBIN,
-                      enabled=True):
+                      enabled=True,
+                      alpn_protocols=constants.
+                      AMPHORA_SUPPORTED_ALPN_PROTOCOLS):
     proto = 'HTTP' if proto is None else proto
     if not tls_enabled:
         tls_ciphers = None
         tls_versions = None
-    monitor_proto = proto if monitor_proto is None else monitor_proto
+        alpn_protocols = None
+    if monitor_proto is None:
+        if proto == constants.PROTOCOL_UDP:
+            monitor_proto = constants.HEALTH_MONITOR_UDP_CONNECT
+        else:
+            monitor_proto = proto
     in_pool = collections.namedtuple(
         'pool', 'id, protocol, lb_algorithm, members, health_monitor, '
                 'session_persistence, enabled, operating_status, '
                 'tls_certificate_id, ca_tls_certificate_id, '
                 'crl_container_id, tls_enabled, tls_ciphers, '
-                'tls_versions, provisioning_status, ' +
+                'tls_versions, provisioning_status, alpn_protocols, ' +
                 constants.HTTP_REUSE)
-    if (proto == constants.PROTOCOL_UDP and
+    if (proto in constants.LVS_PROTOCOLS and
             persistence_type == constants.SESSION_PERSISTENCE_SOURCE_IP):
         kwargs = {'persistence_type': persistence_type,
                   'persistence_timeout': persistence_timeout,
@@ -871,7 +976,8 @@ def sample_pool_tuple(listener_id=None, proto=None, monitor=True,
         tls_enabled=tls_enabled,
         tls_ciphers=tls_ciphers,
         tls_versions=tls_versions,
-        provisioning_status=provisioning_status)
+        provisioning_status=provisioning_status,
+        alpn_protocols=alpn_protocols)
 
 
 def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE',
@@ -951,6 +1057,8 @@ def sample_health_monitor_tuple(proto='HTTP', sample_hm=1,
     if proto == constants.HEALTH_MONITOR_UDP_CONNECT:
         kwargs['check_script_path'] = (CONF.haproxy_amphora.base_path +
                                        'lvs/check/' + 'udp_check.sh')
+    elif proto == lib_consts.HEALTH_MONITOR_SCTP:
+        kwargs['check_script_path'] = 'amphora-health-checker sctp'
     else:
         kwargs['check_script_path'] = None
     return monitor(**kwargs)
@@ -1128,6 +1236,7 @@ def sample_base_expected_config(frontend=None, logging=None, backend=None,
                    "    mode http\n"
                    "    balance roundrobin\n"
                    "    cookie SRV insert indirect nocache\n"
+                   "    load-server-state-from-file global\n"
                    "    timeout check 31s\n"
                    "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
                    "    http-check expect rstatus 418\n"
@@ -1153,7 +1262,7 @@ def sample_base_expected_config(frontend=None, logging=None, backend=None,
                     "    option redispatch\n"
                     "    option splice-request\n"
                     "    option splice-response\n"
-                    "    option http-keep-alive\n\n")
+                    "    option http-keep-alive\n\n\n")
     return ("# Configuration for loadbalancer sample_loadbalancer_id_1\n"
             "global\n"
             "    daemon\n"
@@ -1161,5 +1270,7 @@ def sample_base_expected_config(frontend=None, logging=None, backend=None,
             "    log /run/rsyslog/octavia/log local0\n"
             "    log /run/rsyslog/octavia/log local1 notice\n"
             "    stats socket /var/lib/octavia/sample_loadbalancer_id_1.sock"
-            " mode 0666 level user\n" +
+            " mode 0666 level user\n"
+            "    server-state-file /var/lib/octavia/sample_loadbalancer_id_1"
+            "/servers-state\n" +
             global_opts + defaults + peers + frontend + logging + backend)
