@@ -183,22 +183,28 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIsInstance(create_flow, flow.Flow)
         self.assertIn(constants.LOADBALANCER_ID, create_flow.requires)
         self.assertIn(constants.UPDATE_DICT, create_flow.requires)
+        self.assertIn(constants.BUILD_TYPE_PRIORITY, create_flow.requires)
+        self.assertIn(constants.FLAVOR, create_flow.requires)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, create_flow.requires)
+        self.assertIn(constants.AVAILABILITY_ZONE, create_flow.requires)
+        self.assertIn(constants.SERVER_GROUP_ID, create_flow.requires)
 
         self.assertIn(constants.LISTENERS, create_flow.provides)
+        self.assertIn(constants.SUBNET, create_flow.provides)
         self.assertIn(constants.AMPHORA, create_flow.provides)
         self.assertIn(constants.AMPHORA_ID, create_flow.provides)
         self.assertIn(constants.COMPUTE_ID, create_flow.provides)
         self.assertIn(constants.COMPUTE_OBJ, create_flow.provides)
         self.assertIn(constants.LOADBALANCER, create_flow.provides)
         self.assertIn(constants.DELTAS, create_flow.provides)
-        self.assertIn(constants.ADDED_PORTS, create_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, create_flow.provides)
         self.assertIn(constants.VIP, create_flow.provides)
         self.assertIn(constants.AMP_DATA, create_flow.provides)
+        self.assertIn(constants.SERVER_PEM, create_flow.provides)
         self.assertIn(constants.AMPHORA_NETWORK_CONFIG, create_flow.provides)
 
-        self.assertEqual(6, len(create_flow.requires))
-        self.assertEqual(13, len(create_flow.provides),
-                         create_flow.provides)
+        self.assertEqual(7, len(create_flow.requires))
+        self.assertEqual(13, len(create_flow.provides))
 
     def test_get_create_load_balancer_flows_active_standby_listeners(
             self, mock_get_net_driver):
@@ -218,7 +224,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.COMPUTE_OBJ, create_flow.provides)
         self.assertIn(constants.LOADBALANCER, create_flow.provides)
         self.assertIn(constants.DELTAS, create_flow.provides)
-        self.assertIn(constants.ADDED_PORTS, create_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, create_flow.provides)
         self.assertIn(constants.VIP, create_flow.provides)
         self.assertIn(constants.AMP_DATA, create_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG,
@@ -243,7 +249,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.LOADBALANCER, failover_flow.requires)
         self.assertIn(constants.LOADBALANCER_ID, failover_flow.requires)
 
-        self.assertIn(constants.ADDED_PORTS, failover_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, failover_flow.provides)
         self.assertIn(constants.AMPHORA, failover_flow.provides)
         self.assertIn(constants.AMPHORA_ID, failover_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG,
@@ -275,17 +281,6 @@ class TestLoadBalancerFlows(base.TestCase):
 
         self._test_get_failover_LB_flow_single([amphora_mock])
 
-    def test_get_failover_LB_flow_one_spare_amp_single(self,
-                                                       mock_get_net_driver):
-        amphora_mock = mock.MagicMock()
-        amphora_mock.role = None
-        amphora_mock.lb_network_id = uuidutils.generate_uuid()
-        amphora_mock.compute_id = uuidutils.generate_uuid()
-        amphora_mock.vrrp_port_id = None
-        amphora_mock.vrrp_ip = None
-
-        self._test_get_failover_LB_flow_single([amphora_mock])
-
     def test_get_failover_LB_flow_one_bogus_amp_single(self,
                                                        mock_get_net_driver):
         amphora_mock = mock.MagicMock()
@@ -323,7 +318,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.LOADBALANCER, failover_flow.requires)
         self.assertIn(constants.LOADBALANCER_ID, failover_flow.requires)
 
-        self.assertIn(constants.ADDED_PORTS, failover_flow.provides)
+        self.assertIn(constants.UPDATED_PORTS, failover_flow.provides)
         self.assertIn(constants.AMP_VRRP_INT, failover_flow.provides)
         self.assertIn(constants.AMPHORA, failover_flow.provides)
         self.assertIn(constants.AMPHORA_ID, failover_flow.provides)
@@ -417,24 +412,6 @@ class TestLoadBalancerFlows(base.TestCase):
         self._test_get_failover_LB_flow_no_amps_act_stdby([amphora_mock,
                                                            amphora2_mock])
 
-    def test_get_failover_LB_flow_two_amps_spare_act_stdby(
-            self, mock_get_net_driver):
-        amphora_mock = mock.MagicMock()
-        amphora_mock.role = None
-        amphora_mock.lb_network_id = uuidutils.generate_uuid()
-        amphora_mock.compute_id = uuidutils.generate_uuid()
-        amphora_mock.vrrp_port_id = uuidutils.generate_uuid()
-        amphora_mock.vrrp_ip = '192.0.2.46'
-        amphora2_mock = mock.MagicMock()
-        amphora2_mock.role = constants.ROLE_MASTER
-        amphora2_mock.lb_network_id = uuidutils.generate_uuid()
-        amphora2_mock.compute_id = uuidutils.generate_uuid()
-        amphora2_mock.vrrp_port_id = uuidutils.generate_uuid()
-        amphora2_mock.vrrp_ip = '2001:db8::46'
-
-        self._test_get_failover_LB_flow_no_amps_act_stdby([amphora_mock,
-                                                           amphora2_mock])
-
     def test_get_failover_LB_flow_two_amps_standalone_act_stdby(
             self, mock_get_net_driver):
         amphora_mock = mock.MagicMock()
