@@ -50,6 +50,9 @@ _vip_mock = mock.MagicMock()
 _load_balancer_mock.vip = _vip_mock
 _LB_mock = mock.MagicMock()
 _amphorae_mock = [_amphora_mock]
+_amphora_network_config_mock = mock.MagicMock()
+_amphorae_network_config_mock = {
+    _amphora_mock.id: _amphora_network_config_mock}
 _network_mock = mock.MagicMock()
 _port_mock = mock.MagicMock()
 _ports_mock = [_port_mock]
@@ -74,7 +77,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         conf.config(group="haproxy_amphora",
                     active_connection_max_retries=CONN_MAX_RETRIES)
         conf.config(group="haproxy_amphora",
-                    active_connection_rety_interval=CONN_RETRY_INTERVAL)
+                    active_connection_retry_interval=CONN_RETRY_INTERVAL)
         conf.config(group="controller_worker",
                     loadbalancer_topology=constants.TOPOLOGY_SINGLE)
         self.timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
@@ -379,10 +382,12 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         amphora_post_network_plug_obj = (amphora_driver_tasks.
                                          AmphoraPostNetworkPlug())
-        amphora_post_network_plug_obj.execute(_amphora_mock, _ports_mock)
+        amphora_post_network_plug_obj.execute(_amphora_mock, _ports_mock,
+                                              _amphora_network_config_mock)
 
         (mock_driver.post_network_plug.
-            assert_called_once_with)(_amphora_mock, _port_mock)
+            assert_called_once_with)(_amphora_mock, _port_mock,
+                                     _amphora_network_config_mock)
 
         # Test revert
         amp = amphora_post_network_plug_obj.revert(None, _amphora_mock)
@@ -428,17 +433,20 @@ class TestAmphoraDriverTasks(base.TestCase):
         port_mock = mock.Mock()
         _deltas_mock = {_amphora_mock.id: [port_mock]}
 
-        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock)
+        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock,
+                                              _amphorae_network_config_mock)
 
         (mock_driver.post_network_plug.
-            assert_called_once_with(_amphora_mock, port_mock))
+            assert_called_once_with(_amphora_mock, port_mock,
+                                    _amphora_network_config_mock))
 
         # Test with no ports to plug
         mock_driver.post_network_plug.reset_mock()
 
         _deltas_mock = {'0': [port_mock]}
 
-        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock)
+        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock,
+                                              _amphora_network_config_mock)
         mock_driver.post_network_plug.assert_not_called()
 
         # Test revert
@@ -496,10 +504,7 @@ class TestAmphoraDriverTasks(base.TestCase):
             _session_mock,
             id=AMP_ID,
             status=constants.ERROR)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
+        repo.LoadBalancerRepository.update.assert_not_called()
 
         self.assertIsNone(amp)
 
@@ -513,10 +518,7 @@ class TestAmphoraDriverTasks(base.TestCase):
             _session_mock,
             id=AMP_ID,
             status=constants.ERROR)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
+        repo.LoadBalancerRepository.update.assert_not_called()
 
         self.assertIsNone(amp)
 
@@ -546,33 +548,6 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_driver.post_vip_plug.assert_called_once_with(
             _amphora_mock, _LB_mock, amphorae_net_config_mock)
 
-        # Test revert
-        amp = amphora_post_vip_plug_obj.revert(None, _LB_mock)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
-
-        self.assertIsNone(amp)
-
-        # Test revert with exception
-        repo.LoadBalancerRepository.update.reset_mock()
-        mock_loadbalancer_repo_update.side_effect = Exception('fail')
-        amp = amphora_post_vip_plug_obj.revert(None, _LB_mock)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
-
-        self.assertIsNone(amp)
-
-        # Test revert when this task failed
-        repo.AmphoraRepository.update.reset_mock()
-        amp = amphora_post_vip_plug_obj.revert(
-            failure.Failure.from_exception(Exception('boom')), _amphora_mock,
-            None)
-        repo.AmphoraRepository.update.assert_not_called()
-
     def test_amphora_cert_upload(self,
                                  mock_driver,
                                  mock_generate_uuid,
