@@ -58,6 +58,9 @@ _LB_mock = {
     constants.LOADBALANCER_ID: LB_ID,
 }
 _amphorae_mock = [_db_amphora_mock]
+_amphora_network_config_mock = mock.MagicMock()
+_amphorae_network_config_mock = {
+    _amphora_mock[constants.ID]: _amphora_network_config_mock}
 _network_mock = mock.MagicMock()
 _session_mock = mock.MagicMock()
 
@@ -68,7 +71,7 @@ _session_mock = mock.MagicMock()
 @mock.patch('octavia.db.repositories.ListenerRepository.get',
             return_value=_listener_mock)
 @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
-@mock.patch('octavia.controller.worker.v2.tasks.amphora_driver_tasks.LOG')
+@mock.patch('octavia.controller.worker.v1.tasks.amphora_driver_tasks.LOG')
 @mock.patch('oslo_utils.uuidutils.generate_uuid', return_value=AMP_ID)
 @mock.patch('stevedore.driver.DriverManager.driver')
 class TestAmphoraDriverTasks(base.TestCase):
@@ -79,7 +82,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         conf.config(group="haproxy_amphora",
                     active_connection_max_retries=CONN_MAX_RETRIES)
         conf.config(group="haproxy_amphora",
-                    active_connection_rety_interval=CONN_RETRY_INTERVAL)
+                    active_connection_retry_interval=CONN_RETRY_INTERVAL)
         conf.config(group="controller_worker",
                     loadbalancer_topology=constants.TOPOLOGY_SINGLE)
         self.timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
@@ -221,7 +224,8 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_driver.reload.assert_called_once_with(mock_lb, amphora_mock,
                                                    self.timeout_dict)
         mock_amphora_repo_update.assert_called_once_with(
-            _session_mock, amphora_mock.id, status=constants.ERROR)
+            _session_mock, amphora_mock[constants.ID],
+            status=constants.ERROR)
 
     @mock.patch('octavia.controller.worker.task_utils.TaskUtils.'
                 'mark_listener_prov_status_error')
@@ -380,14 +384,17 @@ class TestAmphoraDriverTasks(base.TestCase):
         amphora_post_network_plug_obj = (amphora_driver_tasks.
                                          AmphoraPostNetworkPlug())
         mock_amphora_repo_get.return_value = _db_amphora_mock
+        fixed_ips = [{constants.SUBNET: {}}]
         port_mock = {constants.NETWORK: mock.MagicMock(),
-                     constants.FIXED_IPS: [mock.MagicMock()],
+                     constants.FIXED_IPS: fixed_ips,
                      constants.ID: uuidutils.generate_uuid()}
-        amphora_post_network_plug_obj.execute(_amphora_mock, [port_mock])
+        amphora_post_network_plug_obj.execute(_amphora_mock, [port_mock],
+                                              _amphora_network_config_mock)
 
         (mock_driver.post_network_plug.
             assert_called_once_with)(_db_amphora_mock,
-                                     network_data_models.Port(**port_mock))
+                                     network_data_models.Port(**port_mock),
+                                     _amphora_network_config_mock)
 
         # Test revert
         amp = amphora_post_network_plug_obj.revert(None, _amphora_mock)
@@ -415,6 +422,39 @@ class TestAmphoraDriverTasks(base.TestCase):
             failure.Failure.from_exception(Exception('boom')), _amphora_mock)
         repo.AmphoraRepository.update.assert_not_called()
 
+    def test_amphora_post_network_plug_with_host_routes(
+            self, mock_driver, mock_generate_uuid, mock_log, mock_get_session,
+            mock_listener_repo_get, mock_listener_repo_update,
+            mock_amphora_repo_get, mock_amphora_repo_update):
+
+        amphora_post_network_plug_obj = (amphora_driver_tasks.
+                                         AmphoraPostNetworkPlug())
+        mock_amphora_repo_get.return_value = _db_amphora_mock
+        host_routes = [{'destination': '10.0.0.0/16',
+                        'nexthop': '192.168.10.3'},
+                       {'destination': '10.2.0.0/16',
+                        'nexthop': '192.168.10.5'}]
+        fixed_ips = [{constants.SUBNET: {'host_routes': host_routes}}]
+
+        port_mock = {constants.NETWORK: mock.MagicMock(),
+                     constants.FIXED_IPS: fixed_ips,
+                     constants.ID: uuidutils.generate_uuid()}
+        amphora_post_network_plug_obj.execute(_amphora_mock, [port_mock],
+                                              _amphora_network_config_mock)
+
+        (mock_driver.post_network_plug.
+            assert_called_once_with)(_db_amphora_mock,
+                                     network_data_models.Port(**port_mock),
+                                     _amphora_network_config_mock)
+
+        call_args = mock_driver.post_network_plug.call_args[0]
+        port_arg = call_args[1]
+        subnet_arg = port_arg.fixed_ips[0].subnet
+        self.assertEqual(2, len(subnet_arg.host_routes))
+        for hr1, hr2 in zip(host_routes, subnet_arg.host_routes):
+            self.assertEqual(hr1['destination'], hr2.destination)
+            self.assertEqual(hr1['nexthop'], hr2.nexthop)
+
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     def test_amphorae_post_network_plug(self, mock_lb_get,
                                         mock_driver,
@@ -439,18 +479,21 @@ class TestAmphoraDriverTasks(base.TestCase):
                      constants.ID: uuidutils.generate_uuid()}
         _deltas_mock = {_db_amphora_mock.id: [port_mock]}
 
-        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock)
+        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock,
+                                              _amphorae_network_config_mock)
 
         (mock_driver.post_network_plug.
          assert_called_once_with(_db_amphora_mock,
-                                 network_data_models.Port(**port_mock)))
+                                 network_data_models.Port(**port_mock),
+                                 _amphora_network_config_mock))
 
         # Test with no ports to plug
         mock_driver.post_network_plug.reset_mock()
 
         _deltas_mock = {'0': [port_mock]}
 
-        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock)
+        amphora_post_network_plug_obj.execute(_LB_mock, _deltas_mock,
+                                              _amphora_network_config_mock)
         mock_driver.post_network_plug.assert_not_called()
 
         # Test revert
@@ -495,7 +538,15 @@ class TestAmphoraDriverTasks(base.TestCase):
                                    mock_amphora_repo_get,
                                    mock_amphora_repo_update):
 
-        amphorae_net_config_mock = mock.MagicMock()
+        amphorae_net_config_mock = {
+            AMP_ID: {
+                constants.VIP_SUBNET: {
+                    'host_routes': []
+                },
+                constants.VRRP_PORT: mock.MagicMock(),
+                'additional_vip_data': []
+            }
+        }
         mock_amphora_repo_get.return_value = _db_amphora_mock
         mock_lb_get.return_value = _db_load_balancer_mock
         amphora_post_vip_plug_obj = amphora_driver_tasks.AmphoraPostVIPPlug()
@@ -509,7 +560,8 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port)
+            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
+            additional_vip_data=[])
 
         # Test revert
         amp = amphora_post_vip_plug_obj.revert(None, _amphora_mock, _LB_mock)
@@ -517,10 +569,7 @@ class TestAmphoraDriverTasks(base.TestCase):
             _session_mock,
             id=AMP_ID,
             status=constants.ERROR)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
+        repo.LoadBalancerRepository.update.assert_not_called()
 
         self.assertIsNone(amp)
 
@@ -534,10 +583,7 @@ class TestAmphoraDriverTasks(base.TestCase):
             _session_mock,
             id=AMP_ID,
             status=constants.ERROR)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
+        repo.LoadBalancerRepository.update.assert_not_called()
 
         self.assertIsNone(amp)
 
@@ -548,6 +594,126 @@ class TestAmphoraDriverTasks(base.TestCase):
             None)
         repo.AmphoraRepository.update.assert_not_called()
 
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    def test_amphora_post_vip_plug_with_host_routes(
+            self, mock_lb_get, mock_loadbalancer_repo_update, mock_driver,
+            mock_generate_uuid, mock_log, mock_get_session,
+            mock_listener_repo_get, mock_listener_repo_update,
+            mock_amphora_repo_get, mock_amphora_repo_update):
+
+        host_routes = [{'destination': '10.0.0.0/16',
+                        'nexthop': '192.168.10.3'},
+                       {'destination': '10.2.0.0/16',
+                        'nexthop': '192.168.10.5'}]
+        amphorae_net_config_mock = {
+            AMP_ID: {
+                constants.VIP_SUBNET: {
+                    'host_routes': host_routes
+                },
+                constants.VRRP_PORT: mock.MagicMock(),
+                'additional_vip_data': []
+            }
+        }
+        mock_amphora_repo_get.return_value = _db_amphora_mock
+        mock_lb_get.return_value = _db_load_balancer_mock
+        amphora_post_vip_plug_obj = amphora_driver_tasks.AmphoraPostVIPPlug()
+        amphora_post_vip_plug_obj.execute(_amphora_mock,
+                                          _LB_mock,
+                                          amphorae_net_config_mock)
+        vip_subnet = network_data_models.Subnet(
+            **amphorae_net_config_mock[AMP_ID]['vip_subnet'])
+        vrrp_port = network_data_models.Port(
+            **amphorae_net_config_mock[AMP_ID]['vrrp_port'])
+
+        mock_driver.post_vip_plug.assert_called_once_with(
+            _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
+            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
+            additional_vip_data=[])
+
+        call_kwargs = mock_driver.post_vip_plug.call_args[1]
+        vip_subnet_arg = call_kwargs.get(constants.VIP_SUBNET)
+        self.assertEqual(2, len(vip_subnet_arg.host_routes))
+        for hr1, hr2 in zip(host_routes, vip_subnet_arg.host_routes):
+            self.assertEqual(hr1['destination'], hr2.destination)
+            self.assertEqual(hr1['nexthop'], hr2.nexthop)
+
+        self.assertEqual(
+            host_routes,
+            amphorae_net_config_mock[AMP_ID][
+                constants.VIP_SUBNET]['host_routes'])
+
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    def test_amphora_post_vip_plug_with_additional_vips(
+            self, mock_lb_get, mock_loadbalancer_repo_update, mock_driver,
+            mock_generate_uuid, mock_log, mock_get_session,
+            mock_listener_repo_get, mock_listener_repo_update,
+            mock_amphora_repo_get, mock_amphora_repo_update):
+
+        host_routes = [{'destination': '10.0.0.0/16',
+                        'nexthop': '192.168.10.3'},
+                       {'destination': '10.2.0.0/16',
+                        'nexthop': '192.168.10.5'}]
+        additional_host_routes = [{'destination': '2001:db9::/64',
+                                   'nexthop': '2001:db8::1:fff'}]
+        amphorae_net_config_mock = {
+            AMP_ID: {
+                constants.VIP_SUBNET: {
+                    'host_routes': host_routes
+                },
+                constants.VRRP_PORT: mock.MagicMock(),
+                'additional_vip_data': [{
+                    'ip_address': '2001:db8::3',
+                    'subnet': {
+                        'host_routes': additional_host_routes
+                    }
+                }]
+            }
+        }
+        mock_amphora_repo_get.return_value = _db_amphora_mock
+        mock_lb_get.return_value = _db_load_balancer_mock
+        amphora_post_vip_plug_obj = amphora_driver_tasks.AmphoraPostVIPPlug()
+        amphora_post_vip_plug_obj.execute(_amphora_mock,
+                                          _LB_mock,
+                                          amphorae_net_config_mock)
+        vip_subnet = network_data_models.Subnet(
+            **amphorae_net_config_mock[AMP_ID]['vip_subnet'])
+        vrrp_port = network_data_models.Port(
+            **amphorae_net_config_mock[AMP_ID]['vrrp_port'])
+        additional_vip_data = [
+            network_data_models.AdditionalVipData(
+                ip_address=add_vip_data['ip_address'],
+                subnet=network_data_models.Subnet(
+                    host_routes=add_vip_data['subnet']['host_routes']))
+            for add_vip_data in amphorae_net_config_mock[
+                AMP_ID]['additional_vip_data']]
+
+        mock_driver.post_vip_plug.assert_called_once_with(
+            _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
+            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
+            additional_vip_data=additional_vip_data)
+
+        call_kwargs = mock_driver.post_vip_plug.call_args[1]
+        vip_subnet_arg = call_kwargs.get(constants.VIP_SUBNET)
+        self.assertEqual(2, len(vip_subnet_arg.host_routes))
+        for hr1, hr2 in zip(host_routes, vip_subnet_arg.host_routes):
+            self.assertEqual(hr1['destination'], hr2.destination)
+            self.assertEqual(hr1['nexthop'], hr2.nexthop)
+
+        self.assertEqual(
+            host_routes,
+            amphorae_net_config_mock[AMP_ID][
+                constants.VIP_SUBNET]['host_routes'])
+
+        add_vip_data_arg = call_kwargs.get('additional_vip_data')
+        self.assertEqual(1, len(add_vip_data_arg[0].subnet.host_routes))
+        hr1 = add_vip_data_arg[0].subnet.host_routes[0]
+        self.assertEqual(
+            additional_host_routes[0]['destination'], hr1.destination)
+        self.assertEqual(
+            additional_host_routes[0]['nexthop'], hr1.nexthop)
+
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     def test_amphorae_post_vip_plug(self, mock_lb_get,
@@ -575,34 +741,8 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port)
-
-        # Test revert
-        amp = amphora_post_vip_plug_obj.revert(None, _LB_mock)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
-
-        self.assertIsNone(amp)
-
-        # Test revert with exception
-        repo.LoadBalancerRepository.update.reset_mock()
-        mock_loadbalancer_repo_update.side_effect = Exception('fail')
-        amp = amphora_post_vip_plug_obj.revert(None, _LB_mock)
-        repo.LoadBalancerRepository.update.assert_called_once_with(
-            _session_mock,
-            id=LB_ID,
-            provisioning_status=constants.ERROR)
-
-        self.assertIsNone(amp)
-
-        # Test revert when this task failed
-        repo.AmphoraRepository.update.reset_mock()
-        amp = amphora_post_vip_plug_obj.revert(
-            failure.Failure.from_exception(Exception('boom')), _amphora_mock,
-            None)
-        repo.AmphoraRepository.update.assert_not_called()
+            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
+            additional_vip_data=[])
 
     def test_amphora_cert_upload(self,
                                  mock_driver,
