@@ -45,6 +45,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
     NETWORK_ID_2 = "10"
     IP_ADDRESS_1 = "10.0.0.2"
     IP_ADDRESS_2 = "12.0.0.2"
+    IPV6_ADDRESS_1 = "2001:db8::1234"
     AMPHORA_ID = "1"
     LB_ID = "2"
     COMPUTE_ID = "3"
@@ -459,6 +460,35 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                           lb, lb.vip, lb.amphorae[0], subnet)
         self.driver.neutron_client.delete_port.assert_called_once()
 
+    def test_plug_aap_port_with_add_vips(self):
+        additional_vips = [
+            {'ip_address': t_constants.MOCK_IP_ADDRESS2,
+             'subnet_id': t_constants.MOCK_VIP_SUBNET_ID2}
+        ]
+        lb = dmh.generate_load_balancer_tree(additional_vips=additional_vips)
+
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
+
+        list_ports = self.driver.neutron_client.list_ports
+        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
+        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
+        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
+        network_attach = self.driver.compute.attach_network_or_port
+        network_attach.side_effect = [t_constants.MOCK_VRRP_INTERFACE1]
+        update_port = self.driver.neutron_client.update_port
+        amp = self.driver.plug_aap_port(lb, lb.vip, lb.amphorae[0], subnet)
+        expected_aap = {
+            'port': {
+                'allowed_address_pairs':
+                    [{'ip_address': lb.vip.ip_address},
+                     {'ip_address': lb.additional_vips[0].ip_address}]}}
+
+        update_port.assert_any_call(amp.vrrp_port_id, expected_aap)
+        self.assertIn(amp.vrrp_ip, [t_constants.MOCK_VRRP_IP1,
+                                    t_constants.MOCK_VRRP_IP2])
+        self.assertEqual(lb.vip.ip_address, amp.ha_ip)
+
     def _set_safely(self, obj, name, value):
         if isinstance(obj, dict):
             current = obj.get(name)
@@ -535,12 +565,13 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             network_id=t_constants.MOCK_NETWORK_ID,
             ip_address=t_constants.MOCK_IP_ADDRESS)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip)
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         self.assertIsInstance(vip, data_models.Vip)
         self.assertEqual(t_constants.MOCK_IP_ADDRESS, vip.ip_address)
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 '_check_extension_enabled', return_value=True)
@@ -568,7 +599,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                                       octavia_owned=True)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -588,6 +619,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 'get_port', side_effect=network_base.PortNotFound)
@@ -611,7 +643,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                                       port_id=t_constants.MOCK_PORT_ID)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -629,6 +661,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 'get_port', side_effect=Exception('boom'))
@@ -668,10 +701,11 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             'network_id': t_constants.MOCK_NETWORK_ID
         }}
         fake_lb_vip = data_models.Vip(subnet_id=t_constants.MOCK_SUBNET_ID,
-                                      network_id=t_constants.MOCK_NETWORK_ID)
+                                      network_id=t_constants.MOCK_NETWORK_ID,
+                                      ip_address=t_constants.MOCK_IP_ADDRESS)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -680,7 +714,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                 'device_owner': allowed_address_pairs.OCTAVIA_OWNER,
                 'admin_state_up': False,
                 'project_id': 'test-project',
-                'fixed_ips': [{'subnet_id': t_constants.MOCK_SUBNET_ID}]
+                'fixed_ips': [{'ip_address': t_constants.MOCK_IP_ADDRESS,
+                               'subnet_id': t_constants.MOCK_SUBNET_ID}]
             }
         }
         create_port.assert_called_once_with(exp_create_port_call)
@@ -689,6 +724,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 '_check_extension_enabled', return_value=True)
@@ -709,7 +745,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                                       ip_address=t_constants.MOCK_IP_ADDRESS)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -728,6 +764,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 '_check_extension_enabled', return_value=True)
@@ -746,7 +783,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         fake_lb_vip = data_models.Vip(network_id=t_constants.MOCK_NETWORK_ID)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -760,6 +797,7 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertIsInstance(vip, data_models.Vip)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertTrue(additional_vips)
 
     @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
                 '_check_extension_enabled', return_value=False)
@@ -776,10 +814,11 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             'network_id': t_constants.MOCK_NETWORK_ID
         }}
         fake_lb_vip = data_models.Vip(subnet_id=t_constants.MOCK_SUBNET_ID,
-                                      network_id=t_constants.MOCK_NETWORK_ID)
+                                      network_id=t_constants.MOCK_NETWORK_ID,
+                                      ip_address=t_constants.MOCK_IP_ADDRESS)
         fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
                                            project_id='test-project')
-        vip = self.driver.allocate_vip(fake_lb)
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
         exp_create_port_call = {
             'port': {
                 'name': 'octavia-lb-1',
@@ -788,7 +827,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                 'device_owner': allowed_address_pairs.OCTAVIA_OWNER,
                 'admin_state_up': False,
                 'tenant_id': 'test-project',
-                'fixed_ips': [{'subnet_id': t_constants.MOCK_SUBNET_ID}]
+                'fixed_ips': [{'ip_address': t_constants.MOCK_IP_ADDRESS,
+                               'subnet_id': t_constants.MOCK_SUBNET_ID}]
             }
         }
         create_port.assert_called_once_with(exp_create_port_call)
@@ -797,8 +837,60 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
 
-    def test_unplug_aap_port_errors_when_update_port_cant_find_port(self):
+    @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
+                '_check_extension_enabled', return_value=False)
+    def test_allocate_vip_with_additional_vips(self, mock_check_ext):
+        port_create_dict = copy.deepcopy(t_constants.MOCK_NEUTRON_PORT)
+        port_create_dict['port']['device_owner'] = (
+            allowed_address_pairs.OCTAVIA_OWNER)
+        port_create_dict['port']['device_id'] = 'lb-1'
+        create_port = self.driver.neutron_client.create_port
+        create_port.return_value = port_create_dict
+        show_subnet = self.driver.neutron_client.show_subnet
+        show_subnet.return_value = {'subnet': {
+            'id': t_constants.MOCK_SUBNET_ID,
+            'network_id': t_constants.MOCK_NETWORK_ID
+        }}
+        fake_lb_vip = data_models.Vip(subnet_id=t_constants.MOCK_SUBNET_ID,
+                                      network_id=t_constants.MOCK_NETWORK_ID,
+                                      ip_address=t_constants.MOCK_IP_ADDRESS)
+        fake_additional_vips = [
+            data_models.AdditionalVip(ip_address=t_constants.MOCK_IP_ADDRESS2),
+            data_models.AdditionalVip(subnet_id=t_constants.MOCK_SUBNET_ID3)]
+        fake_lb = data_models.LoadBalancer(
+            id='1', vip=fake_lb_vip,
+            additional_vips=fake_additional_vips,
+            project_id='test-project')
+        vip, additional_vips = self.driver.allocate_vip(fake_lb)
+        exp_create_port_call = {
+            'port': {
+                'name': 'octavia-lb-1',
+                'network_id': t_constants.MOCK_NETWORK_ID,
+                'device_id': 'lb-1',
+                'device_owner': allowed_address_pairs.OCTAVIA_OWNER,
+                'admin_state_up': False,
+                'tenant_id': 'test-project',
+                'fixed_ips': [
+                    {'ip_address': t_constants.MOCK_IP_ADDRESS,
+                     'subnet_id': t_constants.MOCK_SUBNET_ID},
+                    {'ip_address': t_constants.MOCK_IP_ADDRESS2},
+                    {'subnet_id': t_constants.MOCK_SUBNET_ID3}]
+            }
+        }
+        create_port.assert_called_once_with(exp_create_port_call)
+        self.assertIsInstance(vip, data_models.Vip)
+        self.assertEqual(t_constants.MOCK_IP_ADDRESS, vip.ip_address)
+        self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
+        self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
+        self.assertEqual(fake_lb.id, vip.load_balancer_id)
+        self.assertFalse(additional_vips)
+
+    @mock.patch("time.time")
+    @mock.patch("time.sleep")
+    def test_unplug_aap_port_errors_when_update_port_cant_find_port(
+            self, mock_time_sleep, mock_time_time):
         lb = dmh.generate_load_balancer_tree()
         list_ports = self.driver.neutron_client.list_ports
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
@@ -809,14 +901,22 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         subnet = network_models.Subnet(
             id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
             network_id='3')
-        list_ports.return_value = {'ports': [port1, port2]}
+        list_ports.side_effect = [
+            {'ports': [port1, port2]},
+            {'ports': [port1, port2]},
+            {'ports': [port1]}
+        ]
+        mock_time_time.side_effect = [1, 1, 2]
         update_port = self.driver.neutron_client.update_port
         update_port.side_effect = neutron_exceptions.PortNotFoundClient
         self.assertRaises(network_base.UnplugVIPException,
                           self.driver.unplug_aap_port, lb.vip, lb.amphorae[0],
                           subnet)
 
-    def test_unplug_aap_errors_when_update_port_fails(self):
+    @mock.patch("time.time")
+    @mock.patch("time.sleep")
+    def test_unplug_aap_errors_when_update_port_fails(
+            self, mock_time_sleep, mock_time_time):
         lb = dmh.generate_load_balancer_tree()
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
         port2 = {
@@ -829,7 +929,12 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             network_id='3')
 
         list_ports = self.driver.neutron_client.list_ports
-        list_ports.return_value = {'ports': [port1, port2]}
+        list_ports.side_effect = [
+            {'ports': [port1, port2]},
+            {'ports': [port1, port2]},
+            {'ports': [port1]}
+        ]
+        mock_time_time.side_effect = [1, 1, 2]
 
         update_port = self.driver.neutron_client.update_port
         update_port.side_effect = TypeError
@@ -853,7 +958,12 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.driver.unplug_vip(lb, lb.vip)
         self.assertEqual(len(lb.amphorae), mock.call_count)
 
-    def test_unplug_aap_port(self):
+    @mock.patch("time.time")
+    @mock.patch("time.sleep")
+    @mock.patch("octavia.network.drivers.neutron.allowed_address_pairs."
+                "AllowedAddressPairsDriver.unplug_network")
+    def test_unplug_aap_port(self, mock_unplug_network,
+                             mock_time_sleep, mock_time_time):
         lb = dmh.generate_load_balancer_tree()
         update_port = self.driver.neutron_client.update_port
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
@@ -865,12 +975,19 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             id=t_constants.MOCK_MANAGEMENT_SUBNET_ID,
             network_id='3')
         list_ports = self.driver.neutron_client.list_ports
-        list_ports.return_value = {'ports': [port1, port2]}
+        list_ports.side_effect = [
+            {'ports': [port1, port2]},
+            {'ports': [port1, port2]},
+            {'ports': [port1]}
+        ]
+        mock_time_time.side_effect = [1, 1, 2]
         get_port = self.driver.neutron_client.get_port
         get_port.side_effect = neutron_exceptions.NotFound
         self.driver.unplug_aap_port(lb.vip, lb.amphorae[0], subnet)
         clear_aap = {'port': {'allowed_address_pairs': []}}
         update_port.assert_called_once_with(port2.get('id'), clear_aap)
+        mock_unplug_network.assert_called_once_with(
+            lb.amphorae[0].compute_id, subnet.network_id)
 
     def test_plug_network_when_compute_instance_cant_be_found(self):
         net_id = t_constants.MOCK_NOVA_INTERFACE.net_id
@@ -929,7 +1046,36 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                           self.driver.unplug_network,
                           t_constants.MOCK_COMPUTE_ID, net_id)
 
-    def test_unplug_network(self):
+    @mock.patch("time.time")
+    @mock.patch("time.sleep")
+    def test_unplug_network(self, mock_time_sleep, mock_time_time):
+        list_ports = self.driver.neutron_client.list_ports
+        port1 = t_constants.MOCK_NEUTRON_PORT['port']
+        port2 = {
+            'id': '4', 'network_id': '3', 'fixed_ips':
+            [{'ip_address': '10.0.0.2'}]
+        }
+        list_ports.side_effect = [
+            {'ports': [port1, port2]},
+            {'ports': [port1, port2]},
+            {'ports': [port1]}
+        ]
+        port_detach = self.driver.compute.detach_port
+
+        mock_time_time.side_effect = [1, 1, 2]
+
+        self.driver.unplug_network(t_constants.MOCK_COMPUTE_ID,
+                                   port2.get('network_id'))
+        port_detach.assert_called_once_with(
+            compute_id=t_constants.MOCK_COMPUTE_ID, port_id=port2.get('id'))
+
+        mock_time_sleep.assert_called_once()
+
+    @mock.patch("time.time")
+    @mock.patch("time.sleep")
+    @mock.patch("octavia.network.drivers.neutron.allowed_address_pairs.LOG")
+    def test_unplug_network_timeout(self, mock_log,
+                                    mock_time_sleep, mock_time_time):
         list_ports = self.driver.neutron_client.list_ports
         port1 = t_constants.MOCK_NEUTRON_PORT['port']
         port2 = {
@@ -938,11 +1084,17 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         }
         list_ports.return_value = {'ports': [port1, port2]}
         port_detach = self.driver.compute.detach_port
+
+        mock_time_time.side_effect = [0, 0, 1, 2, 10, 20, 100, 300]
+
         self.driver.unplug_network(t_constants.MOCK_COMPUTE_ID,
                                    port2.get('network_id'))
         port_detach.assert_called_once_with(
             compute_id=t_constants.MOCK_COMPUTE_ID, port_id=port2.get('id'))
 
+        self.assertEqual(6, len(mock_time_sleep.mock_calls))
+        mock_log.warning.assert_called_once()
+
     def test_update_vip(self):
         lc_1 = data_models.ListenerCidr('l1', '10.0.101.0/24')
         lc_2 = data_models.ListenerCidr('l2', '10.0.102.0/24')
@@ -956,7 +1108,11 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                      data_models.Listener(protocol_port=50, peer_port=1026,
                                           protocol=constants.PROTOCOL_UDP)]
         vip = data_models.Vip(ip_address='10.0.0.2')
-        lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
+
+        additional_vip = data_models.AdditionalVip(
+            ip_address=self.IPV6_ADDRESS_1)
+        lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip,
+                                      additional_vips=[additional_vip])
         list_sec_grps = self.driver.neutron_client.list_security_groups
         list_sec_grps.return_value = {'security_groups': [{'id': 'secgrp-1'}]}
         fake_rules = {
@@ -1047,6 +1203,36 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                                       mock.call(expected_create_rule_udp)],
                                      any_order=True)
 
+    def test_update_vip_when_protocol_and_peer_ports_overlap(self):
+        lc_1 = data_models.ListenerCidr('l1', '0.0.0.0/0')
+        listeners = [data_models.Listener(protocol_port=80, peer_port=1024,
+                                          protocol=constants.PROTOCOL_TCP),
+                     data_models.Listener(protocol_port=443, peer_port=1025,
+                                          protocol=constants.PROTOCOL_TCP),
+                     data_models.Listener(protocol_port=1025, peer_port=1026,
+                                          protocol=constants.PROTOCOL_TCP,
+                                          allowed_cidrs=[lc_1])]
+        vip = data_models.Vip(ip_address='10.0.0.2')
+        lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
+        list_sec_grps = self.driver.neutron_client.list_security_groups
+        list_sec_grps.return_value = {'security_groups': [{'id': 'secgrp-1'}]}
+        fake_rules = {
+            'security_group_rules': [
+                {'id': 'rule-80', 'port_range_max': 80, 'protocol': 'tcp'},
+                {'id': 'rule-22', 'port_range_max': 22, 'protocol': 'tcp'}
+            ]
+        }
+        list_rules = self.driver.neutron_client.list_security_group_rules
+        list_rules.return_value = fake_rules
+        delete_rule = self.driver.neutron_client.delete_security_group_rule
+        create_rule = self.driver.neutron_client.create_security_group_rule
+        self.driver.update_vip(lb)
+        delete_rule.assert_called_once_with('rule-22')
+
+        # Create SG rule calls should be 4, each for port 1024/1025/1026/443
+        # No duplicate SG creation for overlap port 1025
+        self.assertEqual(4, create_rule.call_count)
+
     def test_update_vip_when_listener_deleted(self):
         listeners = [data_models.Listener(protocol_port=80,
                                           protocol=constants.PROTOCOL_TCP),
@@ -1255,11 +1441,17 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             t_constants.MOCK_NEUTRON_PORT, t_constants.MOCK_NEUTRON_PORT,
             t_constants.MOCK_NEUTRON_PORT, t_constants.MOCK_NEUTRON_PORT,
             t_constants.MOCK_NEUTRON_PORT, t_constants.MOCK_NEUTRON_PORT,
+            t_constants.MOCK_NEUTRON_PORT, t_constants.MOCK_NEUTRON_PORT,
+            t_constants.MOCK_NEUTRON_PORT, t_constants.MOCK_NEUTRON_PORT,
             Exception('boom')]
         fake_subnet = {'subnet': {
             'id': t_constants.MOCK_SUBNET_ID,
             'gateway_ip': t_constants.MOCK_IP_ADDRESS,
             'cidr': t_constants.MOCK_CIDR}}
+        fake_subnet2 = {'subnet': {
+            'id': t_constants.MOCK_SUBNET_ID2,
+            'gateway_ip': t_constants.MOCK_IP_ADDRESS2,
+            'cidr': t_constants.MOCK_CIDR}}
         show_subnet = self.driver.neutron_client.show_subnet
         show_subnet.return_value = fake_subnet
         configs = self.driver.get_network_configs(load_balancer_mock)
@@ -1294,6 +1486,30 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(expected_subnet_id, config.ha_subnet.id)
         self.assertEqual(expected_subnet_id, config.vrrp_subnet.id)
 
+        # Test with additional_vips
+        load_balancer_mock.additional_vips = [
+            data_models.AdditionalVip(
+                subnet_id=t_constants.MOCK_SUBNET_ID2,
+                ip_address=t_constants.MOCK_IP_ADDRESS2)
+        ]
+        show_subnet.side_effect = [
+            fake_subnet,
+            fake_subnet,
+            fake_subnet,
+            fake_subnet2]
+
+        configs = self.driver.get_network_configs(load_balancer_mock,
+                                                  amphora_mock)
+        self.assertEqual(1, len(configs))
+        config = configs[222]
+        self.assertEqual(t_constants.MOCK_SUBNET_ID2,
+                         config.additional_vip_data[0].subnet.id)
+        self.assertEqual(t_constants.MOCK_IP_ADDRESS2,
+                         config.additional_vip_data[0].ip_address)
+
+        show_subnet.reset_mock(side_effect=True)
+        show_subnet.return_value = fake_subnet
+
         # Test with a specific amphora
         configs = self.driver.get_network_configs(load_balancer_mock,
                                                   amphora_mock)
